@@ -37,6 +37,8 @@
 #define E_NOMEM		23
 #define E_NOSUPPORT	24
 #define E_INVALID	25
+#define E_CLDFAIL	26
+#define E_OLDPORT	27
 
 /* prototypes */
 void print_error(char *msg, ...);
