@@ -19,8 +19,13 @@
 #define FLG_PRIVATE		0x4000
 #define FLG_ERROR		0x2000
 #define FLG_NOPEEP		0x1000  /* for pbbuttonsd internal use only */
+#define FLG_DEPRECIATED		0x0800  /* for pbbuttonsd internal use only */
 
-#define MASK_ERROR		~FLG_ERROR
+/* This mask removes all control bits from a tag identifier so that the pure
+ * identifier is the result. It should always be used before comparing any
+ * tag identifiers.
+ */
+#define TAGMASK			~(FLG_ERROR | FLG_NOPEEP | FLG_DEPRECIATED)
 
 /* Flags for TAG_SYSINFO */
 #define SYSINFO_CONFIGWRITABLE	0x01
