@@ -64,6 +64,9 @@
 #define TAG_AMBIENTLIGHT	MOD_PMAC | 13
 #define TAG_KEYBLIGHTLEVEL	MOD_PMAC | FLG_PRIVATE | 14
 #define TAG_KEYBLIGHTMAX	MOD_PMAC | FLG_PRIVATE | 15
+#define TAG_BATLOG		MOD_PMAC | 16
+#define TAG_BATCYCLE		MOD_PMAC | 17
+#define TAG_BATTERYPRESENT	MOD_PMAC | 18
 
 /* Pri2 Tags */
 #define MOD_DISPLAY		4 << 16
@@ -78,7 +81,7 @@
 #define TAG_KBDILLUMONKEY	MOD_DISPLAY | 19
 #define TAG_KBDILLUMONMOD	MOD_DISPLAY | 20
 #define TAG_LCDAUTOADJUST	MOD_DISPLAY | 12
-#define TAG_BRIGHTNESS		MOD_DISPLAY | 5   /* depreciated */
+#define TAG_BRIGHTNESS		TAG_LCDBRIGHTNESS   /* depreciated */
 #define TAG_LCDBRIGHTNESS	MOD_DISPLAY | 5
 #define TAG_LCDBRIGHTNESSMAX	MOD_DISPLAY | 7
 #define TAG_LCDFADINGSPEED	MOD_DISPLAY | 10
@@ -89,6 +92,7 @@
 #define TAG_LCDAUTOADJMAXAC     MOD_DISPLAY | 28
 #define TAG_KBDAUTOADJUST	MOD_DISPLAY | 13
 #define TAG_KBDBRIGHTNESS	MOD_DISPLAY | 21
+#define TAG_KBDONBRIGHTNESS     MOD_DISPLAY | 29
 #define TAG_KBDBRIGHTNESSMAX	MOD_DISPLAY | 22
 #define TAG_KBDFADINGSPEED	MOD_DISPLAY | 23
 #define TAG_KBDTHRESHOLD	MOD_DISPLAY | 14
@@ -119,7 +123,7 @@
 
 #define MOD_POWER		7 << 16
 #define TAG_SCRIPTBATCRITICAL	MOD_POWER | FLG_STRING | 1 /* depreciated */
-#define TAG_SCRIPTPROFCHANGED	MOD_POWER | FLG_STRING | 2 /* depreciated */
+#define TAG_SCRIPTPROFCHANGED	TAG_SCRIPTPMCS             /* depreciated */
 #define TAG_SCRIPTPMCS		MOD_POWER | FLG_STRING | 2
 #define TAG_SCRIPTHDSETUP	MOD_POWER | FLG_STRING | 3 /* depreciated */
 #define TAG_SLEEPKEY		MOD_POWER | 7
@@ -140,24 +144,40 @@
 #define TAG_NETLOADDEV		MOD_POWER | FLG_STRING | 22
 #define TAG_EMERGENCYACTION	MOD_POWER | 23
 #define TAG_HEARTBEATBEEP	MOD_POWER | 24
-#define TAG_SLEEPKEYSLEEP	MOD_POWER | 25
+#define TAG_SLEEPKEYSLEEP	MOD_POWER | 25  /* depreciated */
 #define TAG_POLICY		MOD_POWER | 26
 
-#define TAG_ONACSLEEP		MOD_POWER | 30
+#define TAG_ONACSLEEP		MOD_POWER | 30  /* depreciated */
 #define TAG_ONACDIM     	MOD_POWER | 31  /* depreciated */
 #define TAG_ONACBLANK     	MOD_POWER | 32  /* depreciated */
-#define TAG_ONACTIMESLEEP	MOD_POWER | 33
-#define TAG_ONACTIMEDIM		MOD_POWER | 34
+#define TAG_ONACTIMESLEEP	MOD_POWER | 33  /* depreciated */
+#define TAG_ONACTIMEDIM		MOD_POWER | 34  /* depreciated */
 #define TAG_ONACTIMEHDOFF	MOD_POWER | 35  /* depreciated */
-#define TAG_ONACCOVERSLEEP	MOD_POWER | 36
-#define TAG_ONACPOLICY		MOD_POWER | 37
+#define TAG_ONACCOVERSLEEP	MOD_POWER | 36  /* depreciated */
+#define TAG_ONACPOLICY		MOD_POWER | 37  /* depreciated */
 
-#define TAG_ONBATTERYSLEEP	MOD_POWER | 40
+#define TAG_ONBATTERYSLEEP	MOD_POWER | 40  /* depreciated */
 #define TAG_ONBATTERYDIM     	MOD_POWER | 41  /* depreciated */
-#define TAG_ONBATTERYBLANK     	MOD_POWER | 41  /* depreciated */
-#define TAG_ONBATTERYTIMESLEEP	MOD_POWER | 43
-#define TAG_ONBATTERYTIMEDIM	MOD_POWER | 44
+#define TAG_ONBATTERYBLANK     	MOD_POWER | 42  /* depreciated */
+#define TAG_ONBATTERYTIMESLEEP	MOD_POWER | 43  /* depreciated */
+#define TAG_ONBATTERYTIMEDIM	MOD_POWER | 44  /* depreciated */
 #define TAG_ONBATTERYTIMEHDOFF	MOD_POWER | 45  /* depreciated */
-#define TAG_ONBATTERYCOVERSLEEP	MOD_POWER | 46
-#define TAG_ONBATTERYPOLICY	MOD_POWER | 47
+#define TAG_ONBATTERYCOVERSLEEP	MOD_POWER | 46  /* depreciated */
+#define TAG_ONBATTERYPOLICY	MOD_POWER | 47  /* depreciated */
+
+#define TAG_ONAC_POLICY		MOD_POWER | 50
+#define TAG_ONAC_TIMERACTION	MOD_POWER | 51
+#define TAG_ONAC_COVERACTION	MOD_POWER | 52
+#define TAG_ONAC_KEYACTION	MOD_POWER | 53
+#define TAG_ONAC_TIMESUSPEND	MOD_POWER | 54
+#define TAG_ONAC_TIMEDIM	MOD_POWER | 55
+
+#define TAG_ONBATT_POLICY	MOD_POWER | 60
+#define TAG_ONBATT_TIMERACTION	MOD_POWER | 61
+#define TAG_ONBATT_COVERACTION	MOD_POWER | 62
+#define TAG_ONBATT_KEYACTION	MOD_POWER | 63
+#define TAG_ONBATT_TIMESUSPEND	MOD_POWER | 64
+#define TAG_ONBATT_TIMEDIM	MOD_POWER | 65
+
 #endif    /* INCLUDE_TAGS_H */
+
