@@ -30,8 +30,9 @@
 
 #define BUFFERLEN	200
 
+int config_no_answer = 0;
+
 static struct tagtable tagtab[] = {
-  { "TAG_SKIP",              TAG_SKIP},
   { "TAG_SAVECONFIG",        TAG_SAVECONFIG},
   { "TAG_AUTORESCAN",        TAG_AUTORESCAN},
   { "TAG_TIMEFORCMD",        TAG_TIMEFORCMD},
@@ -64,6 +65,9 @@ static struct tagtable tagtab[] = {
   { "TAG_TPMODEDOWNMOD",     TAG_TPMODEDOWNMOD},
   { "TAG_TPMODE",            TAG_TPMODE},
   { "TAG_KBDMODE",           TAG_KBDMODE},
+  { "TAG_BATLOG",            TAG_BATLOG},
+  { "TAG_BATTERYPRESENT",    TAG_BATTERYPRESENT},
+  { "TAG_BATCYCLE",          TAG_BATCYCLE},
   { "TAG_AMBIENTLIGHT",      TAG_AMBIENTLIGHT },
   { "TAG_KEYBLIGHTLEVEL",    TAG_KEYBLIGHTLEVEL },
   { "TAG_KEYBLIGHTMAX",      TAG_KEYBLIGHTMAX },
@@ -87,6 +91,7 @@ static struct tagtable tagtab[] = {
   { "TAG_LCDAUTOADJMINAC",   TAG_LCDAUTOADJMINAC },
   { "TAG_LCDAUTOADJMAXAC",   TAG_LCDAUTOADJMAXAC },
   { "TAG_KBDBRIGHTNESS",     TAG_KBDBRIGHTNESS},
+  { "TAG_KBDONBRIGHTNESS",   TAG_KBDONBRIGHTNESS},
   { "TAG_KBDBRIGHTNESSMAX",  TAG_KBDBRIGHTNESSMAX},
   { "TAG_KBDFADINGSPEED",    TAG_KBDFADINGSPEED},
   { "TAG_KBDTHRESHOLD",      TAG_KBDTHRESHOLD },
@@ -115,7 +120,6 @@ static struct tagtable tagtab[] = {
   { "TAG_SLEEPKEY",          TAG_SLEEPKEY },
   { "TAG_SLEEPMOD",          TAG_SLEEPMOD },
   { "TAG_SLEEPKEYDELAY",     TAG_SLEEPKEYDELAY },
-  { "TAG_SLEEPKEYSLEEP",     TAG_SLEEPKEYSLEEP },
   { "TAG_GOTOSLEEP",         TAG_GOTOSLEEP },
   { "TAG_BWLFIRST",          TAG_BWLFIRST },
   { "TAG_BWLSECOND",         TAG_BWLSECOND },
@@ -129,16 +133,32 @@ static struct tagtable tagtab[] = {
   { "TAG_NETLOADPERIOD",     TAG_NETLOADPERIOD },
   { "TAG_NETLOADDEV",        TAG_NETLOADDEV },
   { "TAG_POLICY",            TAG_POLICY },
-  { "TAG_ONACPOLICY",        TAG_ONACPOLICY },
-  { "TAG_ONACSLEEP",         TAG_ONACSLEEP },
-  { "TAG_ONACCOVERSLEEP",    TAG_ONACCOVERSLEEP },
-  { "TAG_ONACTIMESLEEP",     TAG_ONACTIMESLEEP },
-  { "TAG_ONACTIMEDIM",       TAG_ONACTIMEDIM },
-  { "TAG_ONBATTERYPOLICY",   TAG_ONBATTERYPOLICY },
-  { "TAG_ONBATTERYSLEEP",    TAG_ONBATTERYSLEEP },
-  { "TAG_ONBATTERYCOVERSLEEP", TAG_ONBATTERYCOVERSLEEP },
-  { "TAG_ONBATTERYTIMESLEEP", TAG_ONBATTERYTIMESLEEP },
-  { "TAG_ONBATTERYTIMEDIM",  TAG_ONBATTERYTIMEDIM },
+  
+  { "TAG_SLEEPKEYSLEEP",     TAG_SLEEPKEYSLEEP },         /* depreciated */
+  { "TAG_ONACPOLICY",        TAG_ONACPOLICY },            /* depreciated */
+  { "TAG_ONACSLEEP",         TAG_ONACSLEEP },             /* depreciated */
+  { "TAG_ONACCOVERSLEEP",    TAG_ONACCOVERSLEEP },        /* depreciated */
+  { "TAG_ONACTIMESLEEP",     TAG_ONACTIMESLEEP },         /* depreciated */
+  { "TAG_ONACTIMEDIM",       TAG_ONACTIMEDIM },           /* depreciated */
+  { "TAG_ONBATTERYPOLICY",   TAG_ONBATTERYPOLICY },       /* depreciated */
+  { "TAG_ONBATTERYSLEEP",    TAG_ONBATTERYSLEEP },        /* depreciated */
+  { "TAG_ONBATTERYCOVERSLEEP", TAG_ONBATTERYCOVERSLEEP }, /* depreciated */
+  { "TAG_ONBATTERYTIMESLEEP", TAG_ONBATTERYTIMESLEEP },   /* depreciated */
+  { "TAG_ONBATTERYTIMEDIM",  TAG_ONBATTERYTIMEDIM },      /* depreciated */
+
+  { "TAG_ONAC_POLICY",       TAG_ONAC_POLICY },
+  { "TAG_ONAC_TIMERACTION",  TAG_ONAC_TIMERACTION },
+  { "TAG_ONAC_COVERACTION",  TAG_ONAC_COVERACTION },
+  { "TAG_ONAC_KEYACTION",    TAG_ONAC_KEYACTION },
+  { "TAG_ONAC_TIMESUSPEND",  TAG_ONAC_TIMESUSPEND },
+  { "TAG_ONAC_TIMEDIM",      TAG_ONAC_TIMEDIM },
+  { "TAG_ONBATT_POLICY",     TAG_ONBATT_POLICY },
+  { "TAG_ONBATT_TIMERACTION",TAG_ONBATT_TIMERACTION },
+  { "TAG_ONBATT_COVERACTION",TAG_ONBATT_COVERACTION },
+  { "TAG_ONBATT_KEYACTION",  TAG_ONBATT_KEYACTION },
+  { "TAG_ONBATT_TIMESUSPEND",TAG_ONBATT_TIMESUSPEND },
+  { "TAG_ONBATT_TIMEDIM",    TAG_ONBATT_TIMEDIM },
+
   { "TAG_EMERGENCYACTION",   TAG_EMERGENCYACTION },
   { "TAG_HEARTBEATBEEP",     TAG_HEARTBEATBEEP },
   { "TAG_IDENTITY",          TAG_IDENTITY }};
@@ -168,34 +188,33 @@ static struct tagtable tagtab[] = {
 void
 print_usage (char *prgname)
 {
-	printf(_("%s - control client for pbbuttonsd.\n"), prgname);
+	printf(_("%s (version %s) - control client for pbbuttonsd.\n"), prgname, VERSION);
 	printf(_("Usage: %s [-%s] <'query'  | 'config'> <tag> [<data>]\n"), prgname, ARG_ALL);
 	printf (_("Options:\n"
 		"   -%c, --help               display this help and exit\n"
-		"   -%c, --version            display version information and exit\n\n"),
-		ARG_HELP, ARG_VERSION);
+		"   -%c, --version            display version information and exit\n"
+		"   -%c, --ignore             ignore config return and error values\n\n"),
+		ARG_HELP, ARG_VERSION, ARG_IGNORE);
 	print_tags ();
 }
 
 void
 print_tags ()
 {
-	int n;
+	int n, z;
 
 	printf (_("Supported tags:\n"));
 	printf ("    ");
-	for (n=1; n <= (sizeof(tagtab) / sizeof(struct tagtable)); n++) {
+	for (n=1,z=1; n <= (sizeof(tagtab) / sizeof(struct tagtable)); n++) {
 		if (tagtab[n-1].tag & FLG_PRIVATE)
-			printf ("*%-23s", tagtab[n-1].tagname);
+			continue;
 		else
-			printf (" %-23s", tagtab[n-1].tagname);
-		if ((n % 3) == 0)
+			printf ("%-23s", tagtab[n-1].tagname);
+		if ((z++ % 3) == 0)
 			printf ("\n    ");
 	}
 	printf ("\n\n    ");
-	printf (_("The leading 'TAG_' could be omited. Tags marked with an '*' are\n"));
-	printf ("    ");
-	printf (_("private and might give no results.\n"));
+	printf (_("The leading 'TAG_' could be omited.\n"));
 }
 
 int
@@ -204,6 +223,7 @@ evaluate_args(int argc, char *argv[], struct tagitem *taglist)
 	struct option const long_options[] = {
 		  {"help", no_argument, 0, ARG_HELP},
 		  {"version", no_argument, 0, ARG_VERSION},
+		  {"ignore", no_argument, 0, ARG_IGNORE},
 		  {NULL, 0, NULL, 0}
 	};
 	char *prgname;
@@ -220,6 +240,9 @@ evaluate_args(int argc, char *argv[], struct tagitem *taglist)
 				printf(_("pbbcmd, version %s"), VERSION);
 				printf(", (c) 2002 Matthias Grimm\n");
 				return CMD_FLAGS;
+			case ARG_IGNORE:
+				config_no_answer = 1;
+				break;
 			case ARG_HELP:
 			default:
 				print_usage (prgname);
@@ -267,7 +290,7 @@ evaluate_args(int argc, char *argv[], struct tagitem *taglist)
 int
 read_message (char *buffer, int bufferlen)
 {
-	int timeout = 100;
+	int timeout = 40;  /* 4 seconds timeout */
 
 	while (timeout > 0) {
 		if ((ipc_receive (buffer, bufferlen)) >=0) {
@@ -345,7 +368,7 @@ int main(int argc, char *argv[])
 						tags = ((struct pbbmessage *) buffer)->taglist;
 						while (tags->tag != TAG_END) {
 							if (tags->tag & FLG_ERROR) {
-								tagname = identifytagname (tags->tag & MASK_ERROR, "???");
+								tagname = identifytagname (tags->tag & TAGMASK, "???");
 								errname = identifyerror (tags->data, "???");
 								printf (_("Sorry, Couldn't get data for %s: %s."), tagname, _(errname));
 								rc = 1;
@@ -360,12 +383,14 @@ int main(int argc, char *argv[])
 					break;
 				case CMD_CONFIG:
 					ipc_send (0, CHANGEVALUE, taglist);
-					if ((rc = read_message (buffer, BUFFERLEN)) == 0) {
+					if (config_no_answer)
+						rc = 0;
+					else if ((rc = read_message (buffer, BUFFERLEN)) == 0) {
 						pbbmsg = (struct pbbmessage *) buffer;
 						if (pbbmsg->action == CHANGEERROR) {
 							tags = pbbmsg->taglist;
 							while (tags->tag != TAG_END) {
-								tagname = identifytagname (tags->tag & MASK_ERROR, "???");
+								tagname = identifytagname (tags->tag & TAGMASK, "???");
 								errname = identifyerror (tags->data, "???");
 								printf (_("Setting of %s failed: %s.\n"), tagname, _(errname));
 								tags++;
