@@ -15,9 +15,10 @@
 #include <pbbtaglist.h>
 
 /* short options for optarg */
-#define ARG_ALL			"hv"
-#define ARG_HELP			'h'
+#define ARG_ALL			"hvi"
+#define ARG_HELP		'h'
 #define ARG_VERSION		'v'
+#define ARG_IGNORE		'i'
 
 #define CMD_FLAGS		1
 #define CMD_QUERY		2
