@@ -45,51 +45,80 @@
 	    cft_read_int,     cft_write_int,
 	    N_("timeout in seconds for launched scripts, etc.") },
     /* module powersave */
-    { 1, "onAC_policy",       TAG_ONACPOLICY,         0,
+    { 1, "onAC_Policy",       TAG_ONAC_POLICY,          0,
+ 	    cft_read_policy,  cft_write_policy,
+ 	    N_("'"POLICY_NONE_NAME"', '"POLICY_POWERSAVE_NAME"', '"POLICY_USER_NAME"' or '"POLICY_PERFORMANCE_NAME"'") },
+    { 1, "onAC_TimerAction",  TAG_ONAC_TIMERACTION,     0,
+	    cft_read_action,  cft_write_action, 
+	    N_("'"ACTION_NONE_NAME"', '"ACTION_TORAM_NAME"', '"ACTION_TODISK_NAME"' or '"ACTION_BLANK_NAME"'") },
+    { 1, "onAC_CoverAction",  TAG_ONAC_COVERACTION,     0,
+	    cft_read_action,  cft_write_action, 
+	    N_("see TimerAction for possible values") },
+    { 1, "onAC_KeyAction",    TAG_ONAC_KEYACTION,       0,
+	    cft_read_action,  cft_write_action, 
+	    N_("Action (see TimerAction) for the sleepkey") },
+    { 1, "onAC_SuspendTime",  TAG_ONAC_TIMESUSPEND,     0,
+	    cft_read_int,     cft_write_int,
+	    NULL },
+    { 1, "onAC_DimTime",      TAG_ONAC_TIMEDIM,         0,
+	    cft_read_int,     cft_write_int,
+	    NULL },
+	    
+    { 1, "onBattery_policy",  TAG_ONBATT_POLICY,        0,
 	    cft_read_policy,  cft_write_policy,
 	    N_("'"POLICY_NONE_NAME"', '"POLICY_POWERSAVE_NAME"', '"POLICY_USER_NAME"' or '"POLICY_PERFORMANCE_NAME"'") },
+    { 1, "onBattery_TimerAction",TAG_ONBATT_TIMERACTION,0,
+	    cft_read_action,  cft_write_action, 
+	    N_("'"ACTION_NONE_NAME"', '"ACTION_TORAM_NAME"', '"ACTION_TODISK_NAME"' or '"ACTION_BLANK_NAME"'") },
+    { 1, "onBattery_CoverAction",TAG_ONBATT_COVERACTION,0,
+	    cft_read_action,  cft_write_action, 
+	    N_("see TimerAction for possible values") },
+    { 1, "onBattery_KeyAction",TAG_ONBATT_KEYACTION,    0,
+	    cft_read_action,  cft_write_action, 
+	    N_("Action (see TimerAction) for the sleepkey") },
+    { 1, "onBattery_SuspendTime",TAG_ONBATT_TIMESUSPEND,0,
+	    cft_read_int,     cft_write_int,
+	    NULL },
+    { 1, "onBattery_DimTime", TAG_ONBATT_TIMEDIM,       0,
+	    cft_read_int,     cft_write_int,
+	    NULL },
+
+/* --- depreciated config options - read only --- */
     { 1, "onAC_sleep",        TAG_ONACSLEEP,          0,
-	    cft_read_bool,    cft_write_bool,
+	    cft_read_bool,    NULL,
 	    N_("yes=sleep, no=blank after sleep timeout") },
     { 1, "onAC_coversleep",   TAG_ONACCOVERSLEEP,     0,
-	    cft_read_bool,    cft_write_bool,
+	    cft_read_bool,    NULL,
 	    NULL },
     { 1, "onAC_Tsleep",       TAG_ONACTIMESLEEP,      0,
-	    cft_read_int,     cft_write_int,
-	    N_("time in 1/10s") },
+	    cft_read_int,     NULL,
+	    NULL },
     { 1, "onAC_Tdim",         TAG_ONACTIMEDIM,        0,
-	    cft_read_int,     cft_write_int,
-	    N_("time in 1/10s") },
-/*    { 1, "onAC_Thdoff",       TAG_ONACTIMEHDOFF,      0,
-	    cft_read_int,     cft_write_int,
-	    N_("time in multiple of 5 second intervals") }, */
-    { 1, "onBattery_policy",  TAG_ONBATTERYPOLICY,    0,
-	    cft_read_policy,  cft_write_policy,
-	    N_("'"POLICY_NONE_NAME"', '"POLICY_POWERSAVE_NAME"', '"POLICY_USER_NAME"' or '"POLICY_PERFORMANCE_NAME"'") },
+	    cft_read_int,     NULL,
+	    NULL },
     { 1, "onBattery_sleep",   TAG_ONBATTERYSLEEP,     0,
-	    cft_read_bool,    cft_write_bool,
+	    cft_read_bool,    NULL,
 	    NULL },
     { 1, "onBattery_coversleep", TAG_ONBATTERYCOVERSLEEP, 0,
-	    cft_read_bool,    cft_write_bool,
+	    cft_read_bool,    NULL,
 	    NULL },
     { 1, "onBattery_Tsleep",  TAG_ONBATTERYTIMESLEEP, 0,
-	    cft_read_int,     cft_write_int,
-	    N_("time in 1/10s") },
+	    cft_read_int,     NULL,
+	    NULL },
     { 1, "onBattery_Tdim",    TAG_ONBATTERYTIMEDIM,   0,
-	    cft_read_int,     cft_write_int,
-	    N_("time in 1/10s") },
-/*    { 1, "onBattery_Thdoff",  TAG_ONBATTERYTIMEHDOFF, 0,
-	    cft_read_int,     cft_write_int,
-	    N_("time in multiple of 5 second intervals") }, */
+	    cft_read_int,     NULL,
+	    NULL },
+    { 1, "SleepKeySleep",     TAG_SLEEPKEYSLEEP,       0,
+	    cft_read_bool,    NULL,
+	    NULL },
+/* --- */
+
     { 1, "SleepKey",          TAG_SLEEPKEY,           TAG_SLEEPMOD,
 	    cft_read_key,     cft_write_key,
 	    NULL },
     { 1, "SleepKeyDelay",     TAG_SLEEPKEYDELAY,      0,
 	    cft_read_int,     cft_write_int,
 	    N_("values > 0 may be dangerous, if the power key is used to trigger sleep") },
-    { 1, "SleepKeySleep",     TAG_SLEEPKEYSLEEP,       0,
-	    cft_read_bool,    cft_write_bool,
-	    N_("yes=sleep, no=blank screen") },
     { 1, "BWL_first",         TAG_BWLFIRST,           0,
 	    cft_read_int,     cft_write_int,
 	    N_("first battery warnlevel, time in minutes") },
@@ -99,15 +128,9 @@
     { 1, "BWL_last",          TAG_BWLLAST,            0,
 	    cft_read_int,     cft_write_int,
 	    N_("last battery warnlevel, time in minutes") },
-/*    { 1, "Script_BatCritical",TAG_SCRIPTBATCRITICAL,  0,
-	    cft_read_string,  cft_write_string,
-	    NULL }, */
     { 1, "Script_PMCS",       TAG_SCRIPTPMCS,  0,
 	    cft_read_string,  cft_write_string,
 	    NULL },
-/*    { 1, "Script_HDSetup",    TAG_SCRIPTHDSETUP,      0,
-	    cft_read_string,  cft_write_string,
-	    NULL }, */
     { 1, "EmergencyAction",   TAG_EMERGENCYACTION,    0,
 	    cft_read_ema,     cft_write_ema,
 	    N_("action, if battery is critically low") },
@@ -129,7 +152,7 @@
 	    NULL },
     { 1, "NETLoad_min",       TAG_NETLOADMIN,         0,
 	    cft_read_int,     cft_write_int,
-	    N_("trafic in Bytes/s") },
+	    N_("traffic in Bytes/s") },
     { 1, "NETLoad_period",    TAG_NETLOADPERIOD,      0,
 	    cft_read_int,     cft_write_int,
 	    N_("time in seconds") },
@@ -170,6 +193,9 @@
     { 2, "KBD_Brightness",    TAG_KBDBRIGHTNESS,      0,
 	    cft_read_int,     cft_write_int,
 	    N_("initial keyboard illumination level") },
+    { 2, "KBD_OnBrightness",  TAG_KBDONBRIGHTNESS,    0,
+	    cft_read_int,     cft_write_int,
+	    N_("initial level if KBD on/off key is pressed") },
     { 2, "KBD_FadingSpeed",   TAG_KBDFADINGSPEED,     0,
 	    cft_read_int,     cft_write_int,
 	    N_("0 = no smooth fading") },
@@ -250,7 +276,10 @@
 	    N_("'"TRACKPAD_NOTAP_NAME"', '"TRACKPAD_TAP_NAME"', '"TRACKPAD_DRAG_NAME"' or '"TRACKPAD_LOCK_NAME"'")},
     { 5, "KBDMode",           TAG_KBDMODE,            0,
 	    cft_read_kbdmode, cft_write_kbdmode,
-	    N_("'"KEYBOARD_FNBACK_NAME"' or '"KEYBOARD_FNTOP_NAME"'")}
+	    N_("'"KEYBOARD_FNBACK_NAME"' or '"KEYBOARD_FNTOP_NAME"'")},
+    { 5, "Batlog",            TAG_BATLOG,             0,
+	    cft_read_batlogmode, cft_write_batlogmode,
+	    N_("'"BATLOG_NONE_NAME"', '"BATLOG_CYCLE_NAME"' or '"BATLOG_LOG_NAME"'")}
   };
 
 struct tagitem *
@@ -329,26 +358,28 @@ write_configfile (char *configfile)
 
 	section = -1;
 	if ((fd = fopen(configfile, "w"))) {
-		fputs (_("# configfile for pbbuttonsd >= version 0.5\n# for options see man pbbuttonsd.conf\n"), fd);
+		fputs (_("# Configuration file for pbbuttonsd >= version 0.5\n# for options see man pbbuttonsd.conf\n"), fd);
 		for (n=0; n < count; n++) {
 			if (cft[n].section != section) {
 				section = cft[n].section;
 				fputs(sectionhead[section], fd);
 			}
-			if (cft[n].tag1 == (TAG_USERALLOWED)) {
-				if (strlen ((char*) tagfind (taglist, cft[n].tag1, (long) "")) == 0)
-					fputs ("# ", fd);
+			if (cft[n].write != NULL) {
+				if (cft[n].tag1 == (TAG_USERALLOWED)) {
+					if (strlen ((char*) tagfind (taglist, cft[n].tag1, (long) "")) == 0)
+						fputs ("# ", fd);
+				}
+				fprintf (fd, "%-22s= ", cft[n].configtag);
+				fpos1 = ftell(fd);
+				cft[n].write (fd, &cft[n], taglist);
+				if (cft[n].comment) {
+					fpos2 = ftell(fd);
+					if ((fpos2 - fpos1) < 2)
+						fprintf(fd, "\t");
+					fprintf (fd, "\t; %s", _(cft[n].comment));
+				}
+				fputs("\n", fd);
 			}
-			fprintf (fd, "%-20s= ", cft[n].configtag);
-			fpos1 = ftell(fd);
-			cft[n].write (fd, &cft[n], taglist);
-			if (cft[n].comment) {
-				fpos2 = ftell(fd);
-				if ((fpos2 - fpos1) < 2)
-					fprintf(fd, "\t");
-				fprintf (fd, "\t; %s", _(cft[n].comment));
-			}
-			fputs("\n", fd);
 		}
 		fclose(fd);
 	}
@@ -487,6 +518,25 @@ cft_write_kbdmode (FILE *fd, struct cftitem *citem, struct tagitem *taglist)
 	fprintf (fd, "%s", names[val]);	
 }
 
+void
+cft_read_batlogmode (struct cftitem *citem, struct tagitem **taglist, char **strptr, char *arg)
+{
+	char *names[] = {BATLOG_NONE_NAME, BATLOG_CYCLE_NAME, BATLOG_LOG_NAME};
+
+	(*taglist)->tag = citem->tag1;
+	if (((*taglist)->data = read_label (arg, names, BATLOG_LAST)) != -1)
+		(*taglist)++;
+}
+
+void
+cft_write_batlogmode (FILE *fd, struct cftitem *citem, struct tagitem *taglist)
+{
+	char *names[] = {BATLOG_NONE_NAME, BATLOG_CYCLE_NAME, BATLOG_LOG_NAME};
+	int val = tagfind (taglist, citem->tag1, 0);
+	if (val > BATLOG_LAST) val = 0;
+	fprintf (fd, "%s", names[val]);	
+}
+
 void
 cft_read_ema (struct cftitem *citem, struct tagitem **taglist, char **strptr, char *arg)
 {
@@ -525,6 +575,26 @@ cft_write_policy (FILE *fd, struct cftitem *citem, struct tagitem *taglist)
 	fprintf (fd, "%s", names[val]);	
 }
 
+void
+cft_read_action (struct cftitem *citem, struct tagitem **taglist, char **strptr, char *arg)
+{
+	char *names[] = {ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME};
+ 
+	(*taglist)->tag = citem->tag1;
+	if (((*taglist)->data = read_label (arg, names, ACTION_LAST)) != -1)
+		(*taglist)++;
+}
+ 
+void
+cft_write_action (FILE *fd, struct cftitem *citem, struct tagitem *taglist)
+ {
+	char *names[] = {ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME};
+
+	int val = tagfind (taglist, citem->tag1, 0);
+	if (val > ACTION_LAST) val = 0;
+	fprintf (fd, "%s", names[val]);	
+}
+ 
 int
 read_label(char *label, char *names[], int cnt)
 {
