@@ -168,7 +168,7 @@ evaluate_args(struct serverdata *sd, int argc, char *argv[])
 		switch (c) {
 			case ARG_VERSION:
 				printf(_("%s, version %s"), PACKAGE, VERSION);
-				printf(", (c) 2002/2003 Matthias Grimm\n");
+				printf(", (c) 2002-2004 Matthias Grimm\n");
 				return E_INFO;
 			case ARG_DETACH:
 				optarg2 = (optarg) ? optarg : argv[optind];
@@ -240,17 +240,24 @@ ipc_init_stub (struct tagitem *taglist)
 	struct passwd *pwddata;
 
 	err = ipc_init (LIBMODE_SERVER, 0);
-	if (err == E_MSGPORT)
+	switch (err) {
+	   case E_MSGPORT:
 		print_error (_("ERROR: Can't create message port for server: %s.\n"), strerror(errno));
-	else if (err == E_TWICE)
+		break;
+	   case E_TWICE:
 		print_error (_("ERROR: Server is already running. Sorry, only one instance allowed.\n"));
-	else if (err == 0) {
+		break;
+	   case E_OLDPORT:
+		print_error (_("WARNING: Orphaned server port found and removed. All running clients have to be restarted.\n"));
+		err = 0;
+	   case 0:
 		if ((username = tagfind (taglist, TAG_USERALLOWED, -1)) != -1) {
 			if ((pwddata = getpwnam((char *) username)) != NULL)
 				ipc_filteruser(pwddata->pw_uid);
 			else
 				ipc_filterall();      /* switch off IPC-receive */
 		}
+		break;
 	}
 
 	register_function (T10QUEUE, ipc_handler);  /* input via IPC messages */
