@@ -79,7 +79,7 @@ inputmanager_init (struct tagitem *taglist)
 	}
 
 	scan_for_kbdevdevs();
-	if ((fd = open("/dev/input/mice", O_RDONLY)) > 0)  /* open mouse device */
+	if ((fd = open("/dev/input/mice", O_RDONLY)) >= 0)  /* open mouse device */
 		register_inputhandler(fd, mouse_handler);
 
     /* each of TIMERCOUNT (=3) timers have to be initialised here.
@@ -204,7 +204,7 @@ scan_for_kbdevdevs()
 		sprintf(filename, "/dev/input/event%d", n);
 		if ((fd = open(filename, O_RDONLY)) >= 0) {
 			ioctl(fd, EVIOCGBIT(0, EV_MAX), bit);
-		    if (test_bit(EV_KEY, bit) && test_bit(EV_REP, bit)) {
+			if (test_bit(EV_KEY, bit) && test_bit(EV_REP, bit)) {
 				ioctl(fd, EVIOCGID, id);
 				if (id[ID_PRODUCT] != base->evdevs[m].product ||
 					id[ID_VENDOR]  != base->evdevs[m].vendor) {
