@@ -26,6 +26,8 @@ void cdrom_query (struct tagitem *taglist);
 void cdrom_configure (struct tagitem *taglist);
 int cdrom_handle_tags (int cfgure, struct tagitem *taglist);
 char *cdrom_getmountpoint(char *device);
-int cdrom_eject();
+int cdrom_eject ();
+unsigned int cdrom_gettime ();
+void cdrom_testperformance ();
 
 #endif    /* INCLUDE_MODULE_CDROM_H */
