@@ -34,13 +34,15 @@ struct moddata_display {
 	int autoadj_max_bat;
 	int autoadj_min_ac;
 	int autoadj_max_ac;
+
+	int kbdon_brightness;   /* keyboard brightness after switch on */
 	
-	unsigned short keylcdup;     /* keycodes and modifiers */
-	unsigned short modlcdup;     /*     for LCD backlight controls */
+	unsigned short keylcdup;  /* keycodes and modifiers */
+	unsigned short modlcdup;   /*     for LCD backlight controls */
 	unsigned short keylcddn;
 	unsigned short modlcddn;
-	unsigned short keykbdup;     /* keycodes and modifiers */
-	unsigned short modkbdup;     /*     for keyboard illumination controls */
+	unsigned short keykbdup;    /* keycodes and modifiers */
+	unsigned short modkbdup;    /*     for keyboard illumination controls */
 	unsigned short keykbddn;
 	unsigned short modkbddn;
 	unsigned short keykbdon;
@@ -52,22 +54,32 @@ display_init (struct tagitem *taglist)
 {
 	struct moddata_display *base = &modbase_display;
 	struct tagitem args[] = {{ TAG_BACKLIGHTMAX, 0 },
-	                         { TAG_BACKLIGHTLEVEL, 0 },
+	                         { TAG_BACKLIGHTLEVEL, -1 },
 	                         { TAG_KEYBLIGHTMAX, 0 },
 	                         { TAG_AMBIENTLIGHT, -1 },
 	                         { TAG_END, 0 }};
-	int rc;
+	int rc, level;
 
 	static char devbuffer_fb[STDBUFFERLEN];
 	sprintf(devbuffer_fb, DEFAULT_FB_DEVICE);
 
-	process_queue (QUERYQUEUE, args);
+	process_queue (QUERYQUEUE, args); /* get some information from hardware module */
 	base->fbdev  = devbuffer_fb;
 
-	base->lcdlight.current     = (int) tagfind (args, TAG_BACKLIGHTLEVEL, 0);
-	base->lcdlight.target      = (int) tagfind (args, TAG_BACKLIGHTLEVEL, 0);
+	/* if no backlight controller was registered TAG_BACKLIGHTLEVEL
+	 * would return -1 which must be converted to be in brightness
+	 * range. furtheron a flag saves this information.
+	 */
+	base->flags.nobacklight = 0;
+	if ((level = (int) tagfind (args, TAG_BACKLIGHTLEVEL, -1)) == -1) {
+		level = (int) tagfind (args, TAG_BACKLIGHTMAX, 0);
+		base->flags.nobacklight = 1;
+	}
+		
+	base->lcdlight.current     = level;
+	base->lcdlight.target      = level;
 	base->lcdlight.max         = (int) tagfind (args, TAG_BACKLIGHTMAX, 0);
-	base->lcdlight.backup      = (int) tagfind (args, TAG_BACKLIGHTLEVEL, 0);
+	base->lcdlight.backup      = level;
 	base->lcdlight.offset      = 0;
 	base->lcdlight.fadingspeed = 0;
 	base->lcdlight.fadingcount = 0;
@@ -99,18 +111,30 @@ display_init (struct tagitem *taglist)
 	base->autoadj_min_ac  = 1;
 	base->autoadj_max_ac  = base->lcdlight.max;
 	
+	base->kbdon_brightness = KBD_BRIGHTNESS_MIN; /* saved brighhtness for kbd on/off key */
+	
 	base->flags.status 	   = STATUS_NORMAL;
 	base->flags.lmu_enable	   = tagfind(args, TAG_AMBIENTLIGHT, -1) == -1 ? 0 : 1;
 	base->flags.ctrl_fb        = 0;  /* framebuffer disabled by default  */
 	base->flags.dimfullydark   = 0;  /* don't dim to complete darkness by default */
 	base->flags.lcd_autoadjust = 1;  /* lcd and keyboard should react on */
 	base->flags.kbd_autoadjust = 1;  /* ambient light if available       */
-	base->flags.kbd_autoadjust_bak = base->flags.kbd_autoadjust;
 	base->flags.coveropen      = 1;  /* cover open by default same as in laptop module */
 
 	if ((rc = display_handle_tags (MODE_CONFIG, taglist))) /* interprete configfile */
 		return rc;         /* critcal error occured, abort program */
 
+	/* This value must be set to 1 if the initial value of kbd_onbrightness should
+	 * take effect but this will block KBD autoadjustment mode. In KBD autoadjustement
+	 * mode the initial value makes no sense so we set the flag dependent of the
+	 * users autoadjustment settings.
+	 * In case somebody set the keyboard brightness from extern the kbd_switchedoff
+	 * flag will always be 0 except the current brightness is zero (dark). This flag
+	 * status is set by display_handle_tags(), TAG_KBDBRIGHTNESS.
+	 */
+	if (base->kbdlight.current == 0)
+		base->flags.kbd_switchedoff= !base->flags.kbd_autoadjust; 
+
 	register_function (KBDQUEUE, display_keyboard);
 	register_function (QUERYQUEUE, display_query);
 	register_function (CONFIGQUEUE, display_configure);
@@ -147,8 +171,6 @@ display_keyboard (struct tagitem *taglist)
 	struct moddata_display *base = &modbase_display;
 	int code, value, mod, level = 0, changed, tmp;
 
-	static int kbdbackup = KBD_BRIGHTNESS_MIN;  /* for keyboard light on/off key */
-
 	code = (int) tagfind (taglist, TAG_KEYCODE, 0);
 	value = (int) tagfind (taglist, TAG_KEYREPEAT, 0);
 	mod = (int) tagfind (taglist, TAG_MODIFIER, 0);
@@ -161,21 +183,29 @@ display_keyboard (struct tagitem *taglist)
 			level = mod & MOD_SHIFT ? LCD_BRIGHTNESS_MIN : base->lcdlight.current - 1;
 			changed = 1;
 		} else if ((code == base->keykbdup) && ((mod & ~MOD_SHIFT) == base->modkbdup)) {
+			if (base->flags.kbd_switchedoff) {
+				base->kbdlight.current = base->kbdon_brightness;
+				base->flags.kbd_switchedoff = 0;
+			}
 			level = mod & MOD_SHIFT ? base->kbdlight.max : base->kbdlight.current + 1;
 			changed = 2;
 		} else if ((code == base->keykbddn) && ((mod & ~MOD_SHIFT) == base->modkbddn)) {
+			if (base->flags.kbd_switchedoff) {
+				base->kbdlight.current = base->kbdon_brightness;
+				base->flags.kbd_switchedoff = 0;
+			}
 			level = mod & MOD_SHIFT ? KBD_BRIGHTNESS_MIN : base->kbdlight.current - 1;
 			changed = 2;
 		} else if (base->flags.lmu_enable && (code == base->keykbdon) && (mod == base->modkbdon)) {
-			if (base->kbdlight.current != 0) {
-				kbdbackup = base->kbdlight.current;
-				base->flags.kbd_autoadjust = 0;
-				level = KBD_BRIGHTNESS_OFF;
+			if (base->flags.kbd_switchedoff) {
+				level = base->kbdon_brightness;
+				base->flags.kbd_switchedoff = 0;
 			} else {
-				level = kbdbackup ? kbdbackup : KBD_BRIGHTNESS_MIN;
-				base->flags.kbd_autoadjust = base->flags.kbd_autoadjust_bak;
+				base->kbdon_brightness = base->kbdlight.current;
+				base->flags.kbd_switchedoff = 1;
+				level = KBD_BRIGHTNESS_OFF;
 			}
-			level = display_set_brightness(&base->kbdlight, level);
+			level = display_set_kbdbrightness(level);
 			singletag_to_clients(CHANGEVALUE, TAG_KBDBRIGHTNESS, level);
 			changed = 0;
 		} else
@@ -194,7 +224,7 @@ display_keyboard (struct tagitem *taglist)
 					base->lcdlight.offset = -tmp + 1;
 				level = tmp + base->lcdlight.offset;
 			}
-			level = display_set_brightness(&base->lcdlight, level);
+			level = display_set_lcdbrightness(level);
 			singletag_to_clients(CHANGEVALUE, TAG_LCDBRIGHTNESS, level);
 			break;
 		case 2:  /* Keyboard illumination level changed by keys */
@@ -210,7 +240,7 @@ display_keyboard (struct tagitem *taglist)
 						base->kbdlight.offset = -tmp;
 					level = tmp + base->kbdlight.offset;
 				}
-				level = display_set_brightness(&base->kbdlight, level);
+				level = display_set_kbdbrightness(level);
 				singletag_to_clients(CHANGEVALUE, TAG_KBDBRIGHTNESS, level);
 			}
 			break;
@@ -269,7 +299,7 @@ display_timer (struct tagitem *taglist)
  * LCD and keyboard are done.
  * The equations to calculate the new values allow values
  * generated out of range. This doesn't matter as long as
- * the function display_set_brightness() clip them to the
+ * the function display_set_lcdbrightness() clip them to the
  * valid range.
  */
 void
@@ -288,7 +318,7 @@ display_ambienttimer (struct tagitem *taglist)
 		lcdmax = ac ? base->autoadj_max_ac : base->autoadj_max_bat;
 	
 		if ((ambient = process_queue_single (QUERYQUEUE, TAG_AMBIENTLIGHT, -1)) > -1) {
-			if (base->flags.kbd_autoadjust) {
+			if (base->flags.kbd_autoadjust && !base->flags.kbd_switchedoff) {
 				level = -base->kbdlight.max * ambient/base->kbdlight.threshold
 				    + base->kbdlight.max;
 				if (level + base->kbdlight.offset > base->kbdlight.max)
@@ -296,7 +326,7 @@ display_ambienttimer (struct tagitem *taglist)
 				if (level + base->kbdlight.offset < 1)
 					base->kbdlight.offset = -level;
 				level += base->kbdlight.offset; 
-				display_set_brightness(&base->kbdlight, level);
+				display_set_kbdbrightness(level);
 			}
 			
 			if (base->flags.lcd_autoadjust) {
@@ -308,7 +338,7 @@ display_ambienttimer (struct tagitem *taglist)
 				if (level + base->lcdlight.offset < 1)
 					base->lcdlight.offset = -level + 1;
 				level += base->lcdlight.offset; /* offset due to manual adjustment */
-				display_set_brightness(&base->lcdlight, level);
+				display_set_lcdbrightness(level);
 			}
 		}
 	}
@@ -339,10 +369,10 @@ display_handle_tags (int cfgure, struct tagitem *taglist)
 			break;
 		case TAG_LCDBRIGHTNESS:
 			if (cfgure) {
-				level = display_set_brightness (&base->lcdlight, taglist->data);
+				level = display_set_lcdbrightness (taglist->data);
 				singletag_to_clients(CHANGEVALUE, TAG_LCDBRIGHTNESS, level);
 			} else
-				taglist->data = base->lcdlight.current;
+				taglist->data = base->flags.nobacklight ? -1 : base->lcdlight.current;
 			break;
 		case TAG_LCDBRIGHTNESSMAX:
 			if (cfgure)	tagerror (taglist, E_NOWRITE);
@@ -354,11 +384,23 @@ display_handle_tags (int cfgure, struct tagitem *taglist)
 			break;
 		case TAG_KBDBRIGHTNESS:
 			if (cfgure) {
-				level = display_set_brightness (&base->kbdlight, taglist->data);
+				if ((level = display_set_kbdbrightness (taglist->data)) > 0)
+					base->flags.kbd_switchedoff = 0;
 				singletag_to_clients(CHANGEVALUE, TAG_KBDBRIGHTNESS, level);
 			} else
 				taglist->data = base->kbdlight.current;
 			break;
+		case TAG_KBDONBRIGHTNESS:
+			if (cfgure) {
+				level = taglist->data;
+				if (level < KBD_BRIGHTNESS_MIN)
+					level = KBD_BRIGHTNESS_MIN;
+				else if (level > base->kbdlight.max)
+					level = base->kbdlight.max;
+				base->kbdon_brightness = level;
+			} else
+				taglist->data = base->kbdon_brightness;
+			break;
 		case TAG_KBDBRIGHTNESSMAX:
 			if (cfgure)	tagerror (taglist, E_NOWRITE);
 			else		taglist->data = base->kbdlight.max;
@@ -432,7 +474,6 @@ display_handle_tags (int cfgure, struct tagitem *taglist)
 		case TAG_KBDAUTOADJUST:
 			if (cfgure) {
 				base->flags.kbd_autoadjust = taglist->data & 1;
-				base->flags.kbd_autoadjust_bak = base->flags.kbd_autoadjust;
 				base->kbdlight.offset = 0;
 			} else
 				taglist->data = base->flags.kbd_autoadjust;
@@ -475,20 +516,49 @@ display_handle_tags (int cfgure, struct tagitem *taglist)
 			if (cfgure)	base->autoadj_max_ac = taglist->data;
 			else		taglist->data = base->autoadj_max_ac;
 			break;
-		case TAG_COVERSTATUS:  /* private tag */
+		case TAG_COVERSTATUS:      /* private tag */
 			if (cfgure)	base->flags.coveropen = taglist->data & 1;
 			break;
+		case TAG_PREPAREFORSLEEP:  /* private tag */
+			if ((cfgure) && base->flags.nobacklight) {
+				display_framebuffer (0);
+			}
+			break;
+		case TAG_WAKEUPFROMSLEEP:  /* private tag */
+			if ((cfgure) && (base->lcdlight.current))
+				display_framebuffer (1);
+			break;
 		}
 		taglist++;
 	}
 	return rc;
 }
 
-/* This function checks the limits of a given brightness level,
+/* This functions checks the limits of a given brightness level,
  * clip it if neseccary and set it as new target.
  */
 int
-display_set_brightness(struct display_light *illu, int level)
+display_set_lcdbrightness(int level)
+{
+	struct moddata_display *base = &modbase_display;
+	
+	if (base->flags.nobacklight) {
+		if (level > 0) level = base->lcdlight.max;
+		base->lcdlight.target = level;
+		return -1;
+	} else
+		return display_clip_brightness (&base->lcdlight, level);
+}
+
+int
+display_set_kbdbrightness(int level)
+{
+	struct moddata_display *base = &modbase_display;
+	return display_clip_brightness (&base->kbdlight, level);
+}
+
+int
+display_clip_brightness(struct display_light *illu, int level)
 {
 	if (level > illu->max)
 		level = illu->max;
@@ -520,18 +590,24 @@ display_change_brightness(int op)
 
 #if 0
 	level = process_queue_single (QUERYQUEUE, TAG_BACKLIGHTLEVEL, -1));
-	if (level != base->lightsrc[LCD].current) {
+	if (level != base->lcdlight.current) {
 		/* Somebody else has changed the LCD brightness so update
 		 * our system. This should not really happen on a
 		 * pbbuttonsd controlled laptop so it is marked as comment
 		 * until it becomes neseccary.
 		 */
-		base->lightsrc[LCD].current = level;
+		base->lcdlight.current = level == -1 ? 0 : level;
 		base->flags.status = STATUS_NORMAL;
+		base->flags.nobacklight = level == -1 ? 1 : 0;
 	}
 #endif
 
-	if (base->flags.dimfullydark && op == OP_DIM_LIGHT)
+	/* if a backlight controller is not available try to dim to
+	 * complete darkness in hope the framebuffer device is able
+	 * to switch the backlight off. The framebuffer option must
+	 * be configured accordingly to make this work.
+	 */
+	if ((base->flags.dimfullydark || base->flags.nobacklight) && op == OP_DIM_LIGHT)
 		op = OP_DIM_OFF;
 
 	switch (op) {
