@@ -49,14 +49,20 @@ static struct tagtable tagtab[] = {
   { "TAG_BACKLIGHTMAX",      TAG_BACKLIGHTMAX},
   { "TAG_BACKLIGHTLEVEL",    TAG_BACKLIGHTLEVEL},
   { "TAG_COVERSTATUS",       TAG_COVERSTATUS},
+  { "TAG_BATLOG",            TAG_BATLOG},
+  { "TAG_BATCYCLE",          TAG_BATCYCLE},
 #if PEEP_NOCYCLIC
   { "TAG_TIMEREMAINING",     TAG_TIMEREMAINING | FLG_NOPEEP},
   { "TAG_TIMECHANGED",       TAG_TIMECHANGED | FLG_NOPEEP},
   { "TAG_POWERSOURCE",       TAG_POWERSOURCE | FLG_NOPEEP},
+  { "TAG_AMBIENTLIGHT",      TAG_AMBIENTLIGHT | FLG_NOPEEP},
+  { "TAG_BATTERYPRESENT",    TAG_BATTERYPRESENT | FLG_NOPEEP},
 #else
   { "TAG_TIMEREMAINING",     TAG_TIMEREMAINING},
   { "TAG_TIMECHANGED",       TAG_TIMECHANGED},
   { "TAG_POWERSOURCE",       TAG_POWERSOURCE},
+  { "TAG_AMBIENTLIGHT",      TAG_AMBIENTLIGHT},
+  { "TAG_BATTERYPRESENT",    TAG_BATTERYPRESENT},
 #endif
   { "TAG_POWERCHANGED",      TAG_POWERCHANGED},
   { "TAG_REQUESTSLEEP",      TAG_REQUESTSLEEP},
@@ -72,7 +78,6 @@ static struct tagtable tagtab[] = {
   { "TAG_TPMODE",            TAG_TPMODE},
   { "TAG_KBDMODE",           TAG_KBDMODE},
   { "TAG_REPLACEPMUD",       TAG_REPLACEPMUD},
-  { "TAG_AMBIENTLIGHT",      TAG_AMBIENTLIGHT},
   { "TAG_KEYBLIGHTLEVEL",    TAG_KEYBLIGHTLEVEL },
   { "TAG_KEYBLIGHTMAX",      TAG_KEYBLIGHTMAX },
   { "TAG_LCDILLUMUPKEY",     TAG_LCDILLUMUPKEY},
@@ -96,6 +101,7 @@ static struct tagtable tagtab[] = {
   { "TAG_LCDAUTOADJMAXAC",   TAG_LCDAUTOADJMAXAC },
   { "TAG_KBDAUTOADJUST",     TAG_KBDAUTOADJUST },
   { "TAG_KBDBRIGHTNESS",     TAG_KBDBRIGHTNESS},
+  { "TAG_KBDONBRIGHTNESS",   TAG_KBDONBRIGHTNESS},
   { "TAG_KBDBRIGHTNESSMAX",  TAG_KBDBRIGHTNESSMAX},
   { "TAG_KBDFADINGSPEED",    TAG_KBDFADINGSPEED},
   { "TAG_KBDTHRESHOLD",      TAG_KBDTHRESHOLD },
@@ -119,10 +125,8 @@ static struct tagtable tagtab[] = {
   { "TAG_EJECTCDMOD",        TAG_EJECTCDMOD },
   { "TAG_EJECTCDKEYDELAY",   TAG_EJECTCDKEYDELAY },
   { "TAG_EJECTCD",           TAG_EJECTCD },
-  { "-TAG_SCRIPTBATCRITICAL", TAG_SCRIPTBATCRITICAL }, /* obsolete */
-  { "-TAG_SCRIPTPROFCHANGED", TAG_SCRIPTPROFCHANGED},  /* obsolete */
-  { "-TAG_SCRIPTHDSETUP",     TAG_SCRIPTHDSETUP},      /* obsolete */
-  { "TAG_SCRIPTPMCS",        TAG_SCRIPTPMCS},
+  { "TAG_SCRIPTPMCS",        TAG_SCRIPTPMCS },
+  { "TAG_HEARTBEATBEEP",     TAG_HEARTBEATBEEP },
   { "TAG_SLEEPKEY",          TAG_SLEEPKEY },
   { "TAG_SLEEPMOD",          TAG_SLEEPMOD },
   { "TAG_SLEEPKEYDELAY",     TAG_SLEEPKEYDELAY },
@@ -141,24 +145,40 @@ static struct tagtable tagtab[] = {
   { "TAG_NETLOADPERIOD",     TAG_NETLOADPERIOD },
   { "TAG_NETLOADDEV",        TAG_NETLOADDEV },
   { "TAG_POLICY",            TAG_POLICY },
-  { "TAG_ONACPOLICY",        TAG_ONACPOLICY },
-  { "TAG_ONACSLEEP",         TAG_ONACSLEEP },
-  { "-TAG_ONACDIM",          TAG_ONACDIM },          /* obsolete */
-  { "-TAG_ONACBLANK",        TAG_ONACBLANK },        /* obsolete */
-  { "TAG_ONACCOVERSLEEP",    TAG_ONACCOVERSLEEP },
-  { "TAG_ONACTIMESLEEP",     TAG_ONACTIMESLEEP },
-  { "TAG_ONACTIMEDIM",       TAG_ONACTIMEDIM },
-  { "-TAG_ONACTIMEHDOFF",    TAG_ONACTIMEHDOFF },    /* obsolete */
-  { "TAG_ONBATTERYPOLICY",   TAG_ONBATTERYPOLICY },
-  { "TAG_ONBATTERYSLEEP",    TAG_ONBATTERYSLEEP },
-  { "-TAG_ONBATTERYDIM",     TAG_ONBATTERYDIM },       /* obsolete */
-  { "-TAG_ONBATTERYBLANK",   TAG_ONBATTERYBLANK },      /* obsolete */
-  { "TAG_ONBATTERYCOVERSLEEP", TAG_ONBATTERYCOVERSLEEP },
-  { "TAG_ONBATTERYTIMESLEEP", TAG_ONBATTERYTIMESLEEP },
-  { "TAG_ONBATTERYTIMEDIM",  TAG_ONBATTERYTIMEDIM },
-  { "-TAG_ONBATTERYTIMEHDOFF",TAG_ONBATTERYTIMEHDOFF }, /* obsolete */
+  { "TAG_ONAC_POLICY",       TAG_ONAC_POLICY },
+  { "TAG_ONAC_TIMERACTION",  TAG_ONAC_TIMERACTION },
+  { "TAG_ONAC_COVERACTION",  TAG_ONAC_COVERACTION },
+  { "TAG_ONAC_KEYACTION",    TAG_ONAC_KEYACTION },
+  { "TAG_ONAC_TIMESUSPEND",  TAG_ONAC_TIMESUSPEND },
+  { "TAG_ONAC_TIMEDIM",      TAG_ONAC_TIMEDIM },
+  { "TAG_ONBATT_POLICY",     TAG_ONBATT_POLICY },
+  { "TAG_ONBATT_TIMERACTION",TAG_ONBATT_TIMERACTION },
+  { "TAG_ONBATT_COVERACTION",TAG_ONBATT_COVERACTION },
+  { "TAG_ONBATT_KEYACTION",  TAG_ONBATT_KEYACTION },
+  { "TAG_ONBATT_TIMESUSPEND",TAG_ONBATT_TIMESUSPEND },
+  { "TAG_ONBATT_TIMEDIM",    TAG_ONBATT_TIMEDIM },
   { "TAG_EMERGENCYACTION",   TAG_EMERGENCYACTION },
   { "TAG_IDENTITY",          TAG_IDENTITY },
+
+  { "TAG_ONACPOLICY",        TAG_ONACPOLICY         | FLG_DEPRECIATED },
+  { "TAG_ONACSLEEP",         TAG_ONACSLEEP          | FLG_DEPRECIATED },
+  { "TAG_ONACCOVERSLEEP",    TAG_ONACCOVERSLEEP     | FLG_DEPRECIATED },
+  { "TAG_ONACTIMESLEEP",     TAG_ONACTIMESLEEP      | FLG_DEPRECIATED },
+  { "TAG_ONACTIMEDIM",       TAG_ONACTIMEDIM        | FLG_DEPRECIATED },
+  { "TAG_ONBATTERYPOLICY",   TAG_ONBATTERYPOLICY    | FLG_DEPRECIATED },
+  { "TAG_ONBATTERYSLEEP",    TAG_ONBATTERYSLEEP     | FLG_DEPRECIATED },
+  { "TAG_ONBATTERYCOVERSLEEP",TAG_ONBATTERYCOVERSLEEP | FLG_DEPRECIATED },
+  { "TAG_ONBATTERYTIMESLEEP",TAG_ONBATTERYTIMESLEEP | FLG_DEPRECIATED },
+  { "TAG_ONBATTERYTIMEDIM",  TAG_ONBATTERYTIMEDIM   | FLG_DEPRECIATED },
+  { "TAG_ONBATTERYTIMEHDOFF",TAG_ONBATTERYTIMEHDOFF | FLG_DEPRECIATED },
+  { "TAG_ONBATTERYDIM",      TAG_ONBATTERYDIM       | FLG_DEPRECIATED },
+  { "TAG_ONBATTERYBLANK",    TAG_ONBATTERYBLANK     | FLG_DEPRECIATED },
+  { "TAG_ONACTIMEHDOFF",     TAG_ONACTIMEHDOFF      | FLG_DEPRECIATED },
+  { "TAG_ONACDIM",           TAG_ONACDIM            | FLG_DEPRECIATED },
+  { "TAG_ONACBLANK",         TAG_ONACBLANK          | FLG_DEPRECIATED },
+  { "TAG_SCRIPTBATCRITICAL", TAG_SCRIPTBATCRITICAL  | FLG_DEPRECIATED },
+  { "TAG_SCRIPTPROFCHANGED", TAG_SCRIPTPROFCHANGED  | FLG_DEPRECIATED }, 
+  { "TAG_SCRIPTHDSETUP",     TAG_SCRIPTHDSETUP      | FLG_DEPRECIATED },
   { NULL, 0 }};
 
   static struct tagtable errortab[] = {
@@ -309,31 +329,46 @@ void
 peep_taglist (char *source, struct tagitem *taglist)
 {
 	struct tagtable *tt, *tt2;
+	char mark;
 
 	if (taglist != NULL)
 		while (taglist->tag != TAG_END) {
-			tt = findttentry (tagtab,  taglist->tag & MASK_ERROR);
-			if (tt == NULL) {
-				printf (_("unknown tag\n"));
-			} else if (tt->tag & FLG_NOPEEP) {
-				; /* skip tag */
-			} else if (tt->tag == TAG_SKIP) {
-				tt2 = findttentry (tagtab,  taglist->data);
-				if (tt2 != NULL)
-					printf ("%-10s: %-25s: %s                   \n", source, tt->tagname, tt2->tagname);
+			if ((tt = findttentry (tagtab, taglist->tag & TAGMASK))) {
+				if (tt->tag & FLG_NOPEEP) {
+					taglist++;
+					continue;         /* skip tag */
+				}
+
+				if (taglist->tag & FLG_ERROR)
+					mark = 'E';
+				else if (tt->tag & FLG_DEPRECIATED)
+					mark = 'D';
+				else if (tt->tag & FLG_PRIVATE)
+					mark = 'P';
 				else
-					printf ("%-10s: %-25s: ???                   \n", source, tt->tagname);
-			} else if (taglist->tag & FLG_ERROR) {
-				tt2 = findttentry (errortab,  taglist->data);
-				if (tt2 != NULL)
-					printf ("%-10s: %-25s: %s                   \n", source, tt->tagname, tt2->tagname);
-				else
-					printf ("%-10s: %-25s: ???                   \n", source, tt->tagname);
-			} else if (tt->tag & FLG_STRING) {
-					printf ("%-10s: %-25s: \"%s\"                   \n", source, tt->tagname, (char *) taglist->data);
-			} else {
-					printf ("%-10s: %-25s: %ld                    \n", source, tt->tagname, taglist->data);
-			}
+					mark = ' ';
+
+				printf ("%-10s: %c %-25s: ", source, mark, tt->tagname);
+				if (tt->tag == TAG_SKIP) {
+					tt2 = findttentry (tagtab,  taglist->data);
+					if (tt2 != NULL)
+						printf ("%s", tt2->tagname);
+					else
+						printf ("???");
+				} else if (taglist->tag & FLG_ERROR) {
+					tt2 = findttentry (errortab,  taglist->data);
+					if (tt2 != NULL)
+						printf ("%s", tt2->tagname);
+					else
+						printf ("???");
+				} else if (tt->tag & FLG_STRING) {
+						printf ("\"%s\"", (char *) taglist->data);
+				} else {
+						printf ("%ld", taglist->data);
+				}
+				printf ("                    \n");
+			} else
+				printf ("%-10s:   %-25s: %lx\n", source, _("unknown tag"), taglist->tag & TAGMASK);
 			taglist++;
 		}
 }
@@ -342,7 +377,7 @@ struct tagtable *
 findttentry (struct tagtable *ttab, long tag)
 {
 	while (ttab->tag != 0) {
-		if ((ttab->tag & ~FLG_NOPEEP) == tag)
+		if ((ttab->tag & TAGMASK) == tag)
 			return ttab;
 		ttab++;
 	}
