@@ -50,6 +50,7 @@ struct moddata_pmac {
 	int version;     /* PMU version */
 	char *identity;  /* Identity string of this laptop */
 	struct modflags_pmac flags;
+	int flagschanged;        /* PMU flags that have changed recently */
 	int oharevolbutton;      /* level of volume button on OHARE PBs scaled to 0..100 */
 	int oharebrightbutton;   /* level of brightness button on OHARE PBs scaled to 0..15 */
 
@@ -67,14 +68,13 @@ struct moddata_pmac {
 	unsigned short modtpmodedn;
 	int trackpad_mode;
 	int keyboard_mode;             /* keyboard mode for recent keyboards (0xC[0..4]) */
+	int batlog_mode;
+	int batlog_cycle;
 	int i2c_bus_lmu;
 	int lcdfeedback;               /* correction factor for ambient light sensor */
 #if defined(DEBUG) && SIMUAMBIENT
 	int ambient;
 #endif
-#ifdef BATCYCLE
-	int cycle;
-#endif
 } modbase_pmac;
 
 int
@@ -103,6 +103,8 @@ pmac_init (struct tagitem *taglist)
 	base->modtpmodedn	= MOD_ALT;
 	base->trackpad_mode	= -1;
 	base->keyboard_mode	= -1;
+	base->batlog_mode	= BATLOG_NONE;  /* no batlog */
+	base->batlog_cycle	= -1;         /* cycle invalid */
 	base->i2c_bus_lmu       = -1;  /* not available by default */
 	base->timeleft          = -1;
 #if defined(DEBUG) && SIMUAMBIENT
@@ -143,10 +145,21 @@ pmac_init (struct tagitem *taglist)
 	base->identity = identitystring;
 	ioctl(base->fd_pmu, PMU_IOC_CAN_SLEEP, &val);
 	base->flags.sleepsupported = val == 1 ? 1 : 0; /* check if sleep is supported on this system */
-	pmac_update_batteryinfo ();
 
+	pmac_update_flags ();
+	pmac_update_batteryinfo ();
+	base->flagschanged = 0;
+
+	/* Every frame buffer driver could register a backlight controller,
+	 * but only the right one is accepted. Verification is done by checking
+	 * OpenFirmware's property "backlight-control" in register_backlight_controller().
+	 * ioctl(base->fd_pmu, PMU_IOC_GET_BACKLIGHT, &val);  returns -ENODEV if no
+	 * backlight device is registered. This could be used to check if a backlight
+	 * controller is installed or if we have to call our fallback functions.
+	 */
+	
 	if (base->i2c_bus_lmu != -1) {  
-		ioctl(base->fd_pmu, PMU_IOC_GET_BACKLIGHT, &val);
+		val = pmac_get_lcdbacklight();
 		pmac_set_lcdbacklight (0);    /* measure LCD feedback */
 		pmac_set_lcdbacklight (val);  /* restore LCD backlight level */
 	}
@@ -163,10 +176,6 @@ pmac_init (struct tagitem *taglist)
 			base->keyboard_mode = keyboard_get_config(base->fd_adb);	/* get keyboard config*/
 	}
 
-#ifdef BATCYCLE
-	base->cycle = batlog_setup();
-#endif
-
 	if ((register_inputhandler (dup(base->fd_pmu), pmac_pmu_handler)) < 0)
 		print_error(_("WARNING: Can't install PMU input handler. Some functionality may be missing.\n"));
 
@@ -183,14 +192,11 @@ pmac_exit ()
 {
 	struct moddata_pmac *base = &modbase_pmac;
 
-#ifdef BATCYCLE
-	batlog_save();     /* save batlog info on disc */
-#endif
-
+	batlog_save();            /* save batlog info to disc */
 	if (base->fd_pmu != -1)
-		close (base->fd_pmu);		/* close PMU device */
+		close (base->fd_pmu); /* close PMU device */
 	if (base->fd_adb != -1)
-		close (base->fd_adb);		/* close ADB device */
+		close (base->fd_adb); /* close ADB device */
 
 	return 0;
 }
@@ -235,16 +241,16 @@ pmac_secure (struct tagitem *taglist)
 	} else if (base->flags.goto_sleep == 1) {
 		base->flags.goto_sleep = 0;
 		if (base->flags.sleepsupported) {
-#ifdef BATCYCLE
 			batlog_save();
-#endif
 			process_queue_single (CONFIGQUEUE, TAG_PREPAREFORSLEEP, 0);
 #ifdef WITH_PMUD
 			activate_sleepmode ();
+			base->flagschanged = pmac_update_flags ();
 #else
-			do
+			do {
 			    activate_sleepmode ();
-			while (get_cover_position(base->fd_adb) == 0);
+			    base->flagschanged = pmac_update_flags ();
+			} while (base->flags.coveropen == 0);
 #endif
 			base->timeleft = 7200;       /* reset time value so that the */
 				/* filter approximate the real value from top. Otherwise */
@@ -255,9 +261,7 @@ pmac_secure (struct tagitem *taglist)
 				/* recharged during sleep.                           */
 			pmac_update_batteryinfo ();
 			process_queue_single (CONFIGQUEUE, TAG_WAKEUPFROMSLEEP, 0);
-#ifdef BATCYCLE
-			base->cycle = batlog_setup();
-#endif
+			base->batlog_cycle = batlog_setup();
 		}
 	}
 }
@@ -278,7 +282,7 @@ int
 pmac_handle_tags (int cfgure, struct tagitem *taglist)
 {
 	struct moddata_pmac *base = &modbase_pmac;
-	int val, err, rc = 0;
+	int err, rc = 0;
 
 	while (taglist->tag != TAG_END) {
 		switch (taglist->tag) {
@@ -305,12 +309,8 @@ pmac_handle_tags (int cfgure, struct tagitem *taglist)
 			else		taglist->data = BACKLIGHTMAX;
 			break;
 		case TAG_BACKLIGHTLEVEL:   /* private tag */
-			if (cfgure)
-				pmac_set_lcdbacklight (taglist->data);
-			else {
-				ioctl(base->fd_pmu, PMU_IOC_GET_BACKLIGHT, &val);
-				taglist->data = val;
-			}
+			if (cfgure)	pmac_set_lcdbacklight (taglist->data);
+			else		taglist->data = pmac_get_lcdbacklight();
 			break;
 		case TAG_TPMODEUPKEY:
 			if (cfgure)	base->keytpmodeup = taglist->data;
@@ -345,6 +345,18 @@ pmac_handle_tags (int cfgure, struct tagitem *taglist)
 			} else
 				taglist->data = base->keyboard_mode;
 			break;
+		case TAG_BATLOG:
+			if (cfgure) {
+				batlog_save();
+				base->batlog_mode = taglist->data > BATLOG_LAST ? 0 : taglist->data;
+				base->batlog_cycle = batlog_setup();		
+			} else
+				taglist->data = base->batlog_mode;
+			break;
+		case TAG_BATCYCLE:
+			if (cfgure)	tagerror(taglist, E_NOWRITE);
+			else		taglist->data = base->batlog_cycle;
+			break;
 		case TAG_REQUESTSLEEP:    /* private tag */
 			if (cfgure)	base->flags.goto_sleep = 1;   /* trigger delayed sleep */
 			break;
@@ -364,6 +376,17 @@ pmac_handle_tags (int cfgure, struct tagitem *taglist)
 			if (cfgure)	tagerror (taglist, E_NOWRITE);
 			else		taglist->data = base->flags.ac_power;
 			break;
+		case TAG_BATTERYPRESENT:
+			if (cfgure)	tagerror (taglist, E_NOWRITE);
+			/* /proc/pmu is not reliable on machines with removable batteries.
+			 * If no batteries were plugged in, /proc/pmu would not be updated
+			 * anymore and contains the last battery data, which is not valid
+			 * anymore.
+			 * So pbbuttonsd ask the PMU for the existance of any battery and
+			 * report the values from /proc/pmu only if the answer was yes.
+			 */
+			else 		taglist->data = base->flags.batpresent ? base->flags.bat_present : 0;
+			break;
 
 		case TAG_AMBIENTLIGHT:
 			if (base->i2c_bus_lmu < 0)
@@ -404,7 +427,7 @@ pmac_pmu_handler (int fd)
 		case 4:
 			if (n < 3 || intr[2] < 1 || intr[2] > 2)
 				break;
-			call_script ("/sbin/cardctrl", "eject", intr[2]-1);
+			call_script ("/sbin/cardctl %s %d", "eject", intr[2]-1);
 		}
 #if defined(DEBUG) && PMUINTR
 		printf ("\nPMU interrupt: ");
@@ -420,33 +443,28 @@ pmac_timer (struct tagitem *taglist)
 {
 	struct moddata_pmac *base = &modbase_pmac;
 	struct tagitem args[6];
-	int val, val2;
+	int val;
 
 	taglist_init (args);
 
-	val = base->flags.ac_power;
-	val2 = base->timeleft;
+	val = base->timeleft;
+	if (base->flagschanged == 0)     /* check if anybody else has already read the flags */
+		base->flagschanged = pmac_update_flags (); /* if not, get ac power and cover status */
 	pmac_update_batteryinfo ();
-	if (val != base->flags.ac_power) {
+
+	if (base->flagschanged & PMU_ENV_AC_POWER) {
 		taglist_add (args, TAG_POWERCHANGED, base->flags.ac_power);
-#ifdef BATCYCLE
-		if (!base->flags.ac_power)  /* running on Battery */
-			base->cycle++;          /* then increase cycle */
-#endif
+		if (!base->flags.ac_power)      /* running on Battery */
+			base->batlog_cycle++;   /* then increase cycle */
 	}
-	if (val2 != base->timeleft && !base->flags.ac_power) {
+	if (val != base->timeleft && !base->flags.ac_power) {
 		taglist_add (args, TAG_TIMECHANGED, base->timeleft);
-#ifdef BATLOG
 		batlog_write(base->timeleft);
-#endif
 	}
 
 #ifndef WITH_PMUD
-	if ((val = get_cover_position(base->fd_adb)) != -1)
-		if (val != base->flags.coveropen) {
-			base->flags.coveropen = val;
-			taglist_add (args, TAG_COVERSTATUS, base->flags.coveropen);
-		}
+	if (base->flagschanged & PMU_ENV_LID_CLOSED)
+		taglist_add (args, TAG_COVERSTATUS, base->flags.coveropen);
 #endif
 	if (base->version == OHARE_PMU) {         /* PowerBook 3400, etc */
 		if ((val = get_button(base->fd_adb, PMU_GET_VOLBUTTON)) >= 0) {
@@ -469,6 +487,7 @@ pmac_timer (struct tagitem *taglist)
 
 	if (args[0].tag != TAG_END)
 		process_queue (CONFIGQUEUE, args);  /* distribute changes to other modules */
+	base->flagschanged = 0;
 }
 
 
@@ -537,17 +556,6 @@ pmac_update_batteryinfo ()
 	int val, n, syscurrent, time_rem = 0;
 	int charge, chargemax, current, voltage, timeleft[MAX_BATTERIES];
 
-	if ((fd = fopen ("/proc/pmu/info","r"))) {
-		while (fgets (buffer, sizeof (buffer), fd))
-			if ((token = strtok (buffer,":\n"))) {
-				if (!strncmp ("AC Power", token, 8))
-					base->flags.ac_power = atoi (strtok(0,":\n"));
-				else
-					strtok (0,":\n");
-			}
-		fclose(fd);
-	}
-
 	syscurrent = 0;
 	for (n=0; n < MAX_BATTERIES; n++) {
 		charge = chargemax = current = voltage = timeleft[n] = 0;
@@ -613,12 +621,85 @@ pmac_update_batteryinfo ()
 	}
 }
 
+/* use /proc/pmu/info as fallback, if PMU_GET_COVER did't work
+ * As I know this has not happend yet and it is doubtable that
+ * /proc/pmu/info will work correctly in this case. So this
+ * routine may be removed in future versions.
+ */	
+int
+pmac_get_procac ()
+{
+	FILE *fd;
+	char buffer[100], *token;
+	int ac = 0;  /* return battery if anything fails */
+
+	if ((fd = fopen ("/proc/pmu/info","r"))) {
+		while (fgets (buffer, sizeof (buffer), fd))
+			if ((token = strtok (buffer,":\n"))) {
+				if (!strncmp ("AC Power", token, 8))
+					ac = atoi (strtok(0,":\n"));
+				else
+					strtok (0,":\n");
+			}
+		fclose(fd);
+	}
+	return ac;
+}
+
+/* This funtion gets some important flags from the PMU driver. If the
+ * driver doesn't support the ioctl or reports an error, power source
+ * will be read from /proc/pmu/info as fallback, so that power source
+ * should always be set correctly.
+ */
+int
+pmac_update_flags()
+{
+	struct moddata_pmac *base = &modbase_pmac;
+	int env, envold, envnew;
+
+	envold  = base->flags.ac_power ? PMU_ENV_AC_POWER : 0;
+	envold |= base->flags.coveropen ? PMU_ENV_LID_CLOSED : 0;
+	
+	env = get_int_environment(base->fd_adb);
+	if (env != -1) {
+		base->flags.coveropen  = (env & PMU_ENV_LID_CLOSED) ? 0 : 1;
+		base->flags.ac_power   = (env & PMU_ENV_AC_POWER) ? 1 : 0;
+		base->flags.batpresent = (env & PMU_ENV_BATTERY_PRESENT) ? 1 : 0;
+	} else
+		base->flags.ac_power = pmac_get_procac();
+	
+	/* the PB G3 lombard PMU doesn't support the AC power flag in
+	 * environment flags. We use the /proc entry instead.
+	 */
+	if (base->version < 12)
+		base->flags.ac_power = pmac_get_procac();
+		
+	envnew  = base->flags.ac_power ? PMU_ENV_AC_POWER : 0;
+	envnew |= base->flags.coveropen ? PMU_ENV_LID_CLOSED : 0;
+	return (envold ^ envnew);
+}
+
+int
+pmac_get_lcdbacklight ()
+{
+	struct moddata_pmac *base = &modbase_pmac;
+	long val = 0;
+		
+	if ((ioctl(base->fd_pmu, PMU_IOC_GET_BACKLIGHT, &val)) == -1)
+		return -1;
+
+	return (int) val;
+}
+
 void
 pmac_set_lcdbacklight (int val)
 {
 	struct moddata_pmac *base = &modbase_pmac;
 	int ambient;
 	
+	if (val == -1)   /* no backlight controller available */
+		return;
+	
 	ambient = lmu_get_raw_ambient(base->i2c_bus_lmu) - base->lcdfeedback;
 	ioctl(base->fd_pmu, PMU_IOC_SET_BACKLIGHT, &val);
 	base->lcdfeedback = val ? lmu_get_raw_ambient(base->i2c_bus_lmu) - ambient : 0;
@@ -632,6 +713,7 @@ pmac_set_lcdbacklight (int val)
 #endif
 	return;
 }
+
 /* ----------------------------  GENERIC FUNCTIONS ----------------------------- */
 
 void
@@ -871,20 +953,18 @@ get_button(int fd, int button)
 		return -1;      /* not available or invalid */
 }
 
-#ifndef WITH_PMUD
 int
-get_cover_position(int fd)
+get_int_environment(int fd)
 {
 	int count;
 	unsigned char buffer[ADB_BUFSIZE];
 
 	count = send_pmu_request(fd, buffer, 1, PMU_GET_COVER);
 	if (count == 2)
-		return (buffer[1] & PMU_ENV_LID_CLOSED) ? 0 : 1;
+		return buffer[1];
 	else
 		return -1;     /* not available or invalid */
 }
-#endif
 
 /* ----------------------------- LOW LEVEL LMU FUNCTIONS ---------------------------- */
 
@@ -909,7 +989,11 @@ lmu_get_ambient ()
 
 /* This function tries to find the I2C device that controls the keyboard
  * illumination and the ambient light sensor used in alluminum PowerBooks
- * It returns the device number on success, otherwise a negative error code
+ * It returns the device number on success, otherwise a negative error code.
+ * The loop starts with device 4 because that's the correct device on
+ * 17" alluminum PowerBooks. Device 0 could be errornous detected because
+ * the function uses only circumstantial evidence to detect the correct
+ * device.
  */
 int
 lmu_find_bus ()
@@ -920,8 +1004,8 @@ lmu_find_bus ()
 	char i2cdevice[20], buf[4];
 	int n, fd, rc = -E_NOCHAR;
 
-	for(n=0; n < 256; n++) {
-		snprintf(i2cdevice, 19, "/dev/i2c-%d", n );
+	for(n=4; n < 260; n++) {
+		snprintf(i2cdevice, 19, "/dev/i2c-%d", (n & 255) );
 		if ((fd = open (i2cdevice, O_RDWR)) >= 0 ) {
 			if (ioctl (fd, I2C_SLAVE, LMU_ADDR) >= 0 ) {
 				if (read (fd, buf, 4) == 4 ) {
@@ -929,8 +1013,8 @@ lmu_find_bus ()
 					rc = n;
 					break;
 				}
-			} else
-				close (fd);
+			}
+			close (fd);
 		} else if (errno == ENODEV) {
 			rc = -E_NOCHAR;
 			break;
@@ -980,14 +1064,14 @@ lmu_get_raw_ambient (int dev)
 {
 #if defined(DEBUG) && SIMUAMBIENT
 	struct moddata_pmac *base = &modbase_pmac;
-	int lcd, ambient;
+	int ambient, level;
 	int asens[] = {2+2, 182+135, 217+167, 248+200, 309+281,
 	               337+295, 381+341, 423+391, 461+426, 499+473,
 	               538+515, 576+563, 620+669,
 		       1072+1072, 1073+1076, 1077+1080};
 		       
-	ioctl(base->fd_pmu, PMU_IOC_GET_BACKLIGHT, &lcd);
-	ambient = asens[lcd] + base->ambient * 2;
+	level = pmac_get_lcdbacklight();
+	ambient = asens[level == -1 ? 0 : level] + base->ambient * 2;
 	if (ambient > 2*AMBIENTMAX)
 		ambient = 2*AMBIENTMAX;
 	return ambient;
@@ -1011,40 +1095,37 @@ lmu_get_raw_ambient (int dev)
 }
 
 /* --------------------- BATTERY LOGFILE DEBUG FUNCTIONS --------------------- */
-#ifdef BATCYCLE
 int
 batlog_setup()
 {
 	struct moddata_pmac *base = &modbase_pmac;
 	FILE *fd;
 	char buffer[100], *token;
-#ifdef BATLOG
 	char batlog[STDBUFFERLEN];
-#endif
-	int chargeold = 0, chargenew, cycle = 1;
+	int chargeold = 0, chargenew, cycle = -1;
 
-	if ((fd = fopen (DEFAULT_BATCYCLE,"r"))) {
-		while (fgets (buffer, sizeof (buffer), fd))
-			if ((token = strtok (buffer,":\n"))) {
-				if (!strncmp ("cycle", token, 5))
-					cycle = atoi (strtok(0,":\n"));
-				else if (!strncmp ("charge", token, 6))
-					chargeold = atoi(strtok(0, ":\n"));
-				else
-					strtok (0,":\n");
+	if (base->batlog_mode) {
+		cycle = 1;
+		if ((fd = fopen (DEFAULT_BATCYCLE,"r"))) {
+			while (fgets (buffer, sizeof (buffer), fd))
+				if ((token = strtok (buffer,":\n"))) {
+					if (!strncmp ("cycle", token, 5))
+						cycle = atoi (strtok(0,":\n"));
+					else if (!strncmp ("charge", token, 6))
+						chargeold = atoi(strtok(0, ":\n"));
+					else
+						strtok (0,":\n");
+				}
+			fclose(fd);
+			if (!base->flags.ac_power) {
+				chargenew = base->charge[0] + base->charge[1];
+				if ((chargenew-chargeold) > 50)
+					cycle++;  /* start new cycle */
 			}
-		fclose(fd);
-		if (!base->flags.ac_power) {
-			chargenew = base->charge[0] + base->charge[1];
-			if ((chargenew-chargeold) > 50)
-				cycle++;  /* start new cycle */
 		}
-#ifdef BATLOG
 		snprintf (batlog, STDBUFFERLEN, DEFAULT_BATLOG, cycle);
-		print_error(_("INFO: Battery logfile %s active!\n"), batlog);
-#else
-		print_error(_("INFO: Current battery cycle: %d.\n"), cycle);
-#endif
+		print_error(_("INFO: Current battery cycle: %d, active logfile: %s.\n"),
+			cycle, base->batlog_mode == BATLOG_LOG ? batlog : "none");
 	}
 	return cycle;
 }
@@ -1055,14 +1136,14 @@ batlog_save()
 	struct moddata_pmac *base = &modbase_pmac;
 	FILE *fd;
 
-	if ((fd = fopen (DEFAULT_BATCYCLE,"w"))) {
-		fprintf (fd,"cycle   : %d\n", base->cycle);
-		fprintf (fd,"charge  : %d\n", base->charge[0] + base->charge[1]);
-		fclose(fd);
-	}
+	if (base->batlog_mode)
+		if ((fd = fopen (DEFAULT_BATCYCLE,"w"))) {
+			fprintf (fd,"cycle   : %d\n", base->batlog_cycle);
+			fprintf (fd,"charge  : %d\n", base->charge[0] + base->charge[1]);
+			fclose(fd);
+		}
 }
-#endif /* BATCYCLE */
-#ifdef BATLOG
+
 void
 batlog_write(long timerem)
 {
@@ -1073,17 +1154,20 @@ batlog_write(long timerem)
 	static long curtime = 0;
 	int rc;
 
-	snprintf (batlog, STDBUFFERLEN, DEFAULT_BATLOG, base->cycle);
+	if (base->batlog_mode != BATLOG_LOG)
+		return;
+		
+	snprintf (batlog, STDBUFFERLEN, DEFAULT_BATLOG, base->batlog_cycle);
 	rc = check_devorfile(batlog, TYPE_FILE);
 	if (rc == E_NOEXIST) {
 		if ((fd = fopen(batlog, "w")) != NULL) { /* open new protocol file */
-			fprintf(fd, "# battery cycle %d\n# time\tvoltage[0]\tcharge[0]\tchargemax[0]\tvoltage[1]\tcharge[1]\tchargemax[1]\ttimerem.\n", base->cycle);
+			fprintf(fd, "# battery cycle %d\n# time\tvoltage[0]\tcharge[0]\tchargemax[0]\tvoltage[1]\tcharge[1]\tchargemax[1]\ttimerem.\n", base->batlog_cycle);
 			fclose(fd);
 			rc = 0;
 		} else
 			rc = E_OPEN;
 	}
-
+		
 	/* to evaluate battery degradation current battery values
 	   are written to a lofile for further investigation.
 	   Each battery charge/discharge cycle in a seperate file. */
@@ -1101,5 +1185,4 @@ batlog_write(long timerem)
 		}
 	}
 }
-#endif  /* BATLOG */
 #endif /* WITH_MODULE_PMAC */
