@@ -85,60 +85,59 @@ power_init (struct tagitem *taglist)
 
 	power_sync(); /* syncronise some data with module_pmac */
 
-	scriptbuffer1[0]         = 0; /* no default for script_pmcs */
-	base->script_pmcs        = scriptbuffer1;
-	base->mode               = MODE_AWAKE;
+	scriptbuffer1[0]            = 0; /* no default for script_pmcs */
+	base->script_pmcs           = scriptbuffer1;
+	base->mode                  = MODE_AWAKE;
 
-	base->policy             = POLICY_PERFORMANCE;
-	base->flags.usersetpolicy= 0;
+	base->policy                = POLICY_PERFORMANCE;
+	base->flags.usersetpolicy   = 0;
 	
 	/* default profile for running on AC power */
-	base->onAC.flags.sleep      = base->flags.sleep_supported;
-	base->onAC.flags.coversleep = base->flags.sleep_supported;
-	base->onAC.Tsleep           = AC_TIME_SLEEP;
-	base->onAC.Tdim             = AC_TIME_DIM;
-	base->onAC.Thdoff           = AC_TIME_HDOFF;
 	base->onAC.policy	    = POLICY_PERFORMANCE;
-
+	base->onAC.timeraction      = base->flags.sleep_supported ? ACTION_TORAM : ACTION_BLANK;
+	base->onAC.coveraction      = base->flags.sleep_supported ? ACTION_TORAM : ACTION_BLANK;
+	base->onAC.keyaction        = base->flags.sleep_supported ? ACTION_TORAM : ACTION_BLANK;
+	base->onAC.suspendtime      = AC_TIME_SLEEP;
+	base->onAC.dimtime          = AC_TIME_DIM;
+ 	
 	/* default profile for running on battery */
-	base->onBattery.flags.sleep      = base->flags.sleep_supported;
-	base->onBattery.flags.coversleep = base->flags.sleep_supported;
-	base->onBattery.Tsleep           = BATTERY_TIME_SLEEP;
-	base->onBattery.Tdim             = BATTERY_TIME_DIM;
-	base->onBattery.Thdoff           = BATTERY_TIME_HDOFF;
-	base->onBattery.policy	         = POLICY_POWERSAVE;
-	
+	base->onBattery.policy	    = POLICY_POWERSAVE;
+	base->onBattery.timeraction = base->flags.sleep_supported ? ACTION_TORAM : ACTION_BLANK;
+	base->onBattery.coveraction = base->flags.sleep_supported ? ACTION_TORAM : ACTION_BLANK;
+	base->onBattery.keyaction   = base->flags.sleep_supported ? ACTION_TORAM : ACTION_BLANK;
+	base->onBattery.suspendtime = BATTERY_TIME_SLEEP;
+	base->onBattery.dimtime     = BATTERY_TIME_DIM;
+
 	/* definitions for immediatly sleep function */
-	base->keysleep           = KEY_POWER;
-	base->modsleep           = MOD_NONE;
-	base->sleepkeydelaytime  = 0;         /* time delay till reaction */
-	base->sleeptime          = 0;        /* counter for time till sleep */
-	base->sleepcountdown     = 60;      /* counter for last 5 seconds before sleep */
-	base->flags.sleepkeysleep= base->flags.sleep_supported;
+	base->keysleep              = KEY_POWER;
+	base->modsleep              = MOD_NONE;
+	base->sleepkeydelaytime     = 0;       /* time delay till reaction */
+	base->sleeptime             = 0;      /* counter for time till sleep */
+	base->sleepcountdown        = 60;    /* counter for last 5 seconds before sleep */
 
 	/* definitions for battery supervising functions */
-	base->BWL1              = BAT_WARNLEVEL1;
-	base->BWL2              = BAT_WARNLEVEL2;
-	base->BWL3              = BAT_WARNLEVEL3;
-	base->warnlevel         = 0;          /* current battery warnlevel */
-	base->emergency         = base->flags.sleep_supported ? EMA_SLEEP : EMA_SIGNAL; /* default emergency action */
-	base->flags.sigpwr_sent = 0;  /* currently no sigpwr in progress */
+	base->BWL1                  = BAT_WARNLEVEL1;
+	base->BWL2                  = BAT_WARNLEVEL2;
+	base->BWL3                  = BAT_WARNLEVEL3;
+	base->warnlevel             = 0;          /* current battery warnlevel */
+	base->emergency             = base->flags.sleep_supported ? EMA_SLEEP : EMA_SIGNAL; /* default emergency action */
+	base->flags.sigpwr_sent     = 0;  /* currently no sigpwr in progress */
 
 	/* definitions for sleeplocks */
-	base->flags.cpuload_enable = 0;       /* disabled by default */
-	base->flags.sleeplock_cpu  = 0;       /* lock open */
-	base->cpuload_min          = 20;      /* lock closed if cpuload <20% */
-	base->cpuload_period       = 60;      /*  for >60s, if enabled */
-	base->flags.netload_enable = 0;       /* disabled by default */
-	base->flags.sleeplock_net  = 0;       /* lock open */
-	base->netload_min          = 4096;    /* lock closed if netload <4 KB/s */
-	base->netload_period       = 60;      /*  for >60s, if enabled  */
+	base->flags.cpuload_enable  = 0;    /* disabled by default */
+	base->flags.sleeplock_cpu   = 0;    /* lock open */
+	base->cpuload_min           = 20;   /* lock closed if cpuload <20% */
+	base->cpuload_period        = 60;   /*  for >60s, if enabled */
+	base->flags.netload_enable  = 0;    /* disabled by default */
+	base->flags.sleeplock_net   = 0;    /* lock open */
+	base->netload_min           = 4096; /* lock closed if netload <4 KB/s */
+	base->netload_period        = 60;   /*  for >60s, if enabled  */
 	sprintf(scriptbuffer2, DEFAULT_NETLOAD_DEV);
-	base->netload_dev          = scriptbuffer2;
+	base->netload_dev           = scriptbuffer2;
 
-	base->flags.heartbeat = 0;     /* heartbeat beep disabled */
-	base->flags.heartbeat_enable = 1;  /* heartbeat beep is possible by default */ 
-	base->flags.coveropen = 1;  /* cover open by default same as in laptop module */
+	base->flags.heartbeat       = 0;  /* heartbeat beep disabled */
+	base->flags.heartbeat_enable= 1;  /* heartbeat beep is possible by default */ 
+	base->flags.coveropen       = 1;  /* cover open by default same as in laptop module */
 
 	if ((rc = power_handle_tags (MODE_CONFIG, taglist))) /* interprete configfile */
 		return rc;         /* critcal error occured, abort program */
@@ -150,9 +149,7 @@ power_init (struct tagitem *taglist)
 				  send a TAG_POWERCHANGE to correct it and start the script
 				  Not elegant but works */
 	
-	ipc_protect_tag (TAG_SCRIPTBATCRITICAL);     /* tags only for privileged use */
-	ipc_protect_tag (TAG_SCRIPTPROFCHANGED);
-	ipc_protect_tag (TAG_SCRIPTHDSETUP);
+	ipc_protect_tag (TAG_SCRIPTPMCS);  /* tags only for privileged use */
 
 	register_function (KBDQUEUE, power_keyboard);
 	register_function (MOUSEQUEUE, power_mouse);
@@ -196,10 +193,8 @@ power_keyboard (struct tagitem *taglist)
 		if ((code == base->keysleep) && (mod == base->modsleep))
 			if ((base->sleepkeydelaytime == 0) || (keydelayms(&base->tv, value, base->sleepkeydelaytime)))
 				if (base->flags.sleeptriggered == 0) {
-					base->flags.sleeptriggered = 1;     /* sleep should be triggered only once */
-					singletag_to_clients(WARNING, TAG_SLEEPINSECONDS, 0);
-					sleep(1);   /* give clients a chance to react */
-					power_trigger_sleep (base->flags.sleepkeysleep);
+					base->flags.sleeptriggered = 1; /* sleep should be triggered only once */
+					power_suspend (base->activeProfile->keyaction);
 				}
 }
 
@@ -259,41 +254,25 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 		switch (taglist->tag) {
 		case TAG_REINIT:
 			/* set power policy again after resume from suspend
-			 * to disc. */
+			 * to disk. */
 			power_setpolicy(taglist->data, base->powersource);
 			break;
 		case TAG_SCRIPTPMCS:
 			if (cfgure) {
-				if ((check_script ((char *) taglist->data, "ss")) == 0) {
+				if ((check_script ((char *) taglist->data, "sss")) == 0) {
 					if ((err = copy_path ((char *) taglist->data, base->script_pmcs, TYPE_FILE, CPFLG_PBBONLY)))
 						rc = tagerror (taglist, err);
 				} else {
-					print_error(_("ERROR: Too many formatsigns. Max two %%s allowed in TAG_SCRIPTPMCS.\n"));
+					print_error(_("ERROR: Too many formatsigns. Max three %%s allowed in TAG_SCRIPTPMCS.\n"));
 					rc = tagerror (taglist, E_FORMAT);
 				}
 			} else
 				taglist->data = (long) base->script_pmcs;
 			break;
-		case TAG_ONACSLEEP:
-			if (cfgure) {
-				if (base->flags.sleep_supported)
-					base->onAC.flags.sleep = taglist->data;
-			} else
-				taglist->data = base->onAC.flags.sleep;
-			break;
-		case TAG_ONACCOVERSLEEP:
-			if (cfgure)	base->onAC.flags.coversleep = taglist->data;
-			else		taglist->data = base->onAC.flags.coversleep;
-			break;
-		case TAG_ONACTIMESLEEP:
-			if (cfgure)	base->onAC.Tsleep = taglist->data;
-			else		taglist->data = base->onAC.Tsleep;
-			break;
-		case TAG_ONACTIMEDIM:
-			if (cfgure)	base->onAC.Tdim = taglist->data;
-			else		taglist->data = base->onAC.Tdim;
-			break;
-		case TAG_ONACPOLICY:
+			
+		/* Tags for power profiles */	
+		case TAG_ONAC_POLICY:
+ 		case TAG_ONACPOLICY:      /* depreceated */
 			if (cfgure) {
 				if (taglist->data > POLICY_LAST)
 					tagerror(taglist, E_INVALID);
@@ -306,26 +285,54 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 			} else
 				taglist->data = base->onAC.policy;
 			break;
-		case TAG_ONBATTERYSLEEP:
+		case TAG_ONAC_TIMERACTION:
+ 			if (cfgure) {
+				if (taglist->data > ACTION_LAST)
+					tagerror(taglist, E_INVALID);
+				else
+					base->onAC.timeraction = taglist->data;
+ 			} else
+				taglist->data = base->onAC.timeraction;
+ 			break;
+		case TAG_ONAC_COVERACTION:
 			if (cfgure) {
-				if (base->flags.sleep_supported)
-					base->onBattery.flags.sleep = taglist->data;
+				if (taglist->data > ACTION_LAST)
+					tagerror(taglist, E_INVALID);
+				else
+					base->onAC.coveraction = taglist->data;
 			} else
-				taglist->data = base->onBattery.flags.sleep;
+				taglist->data = base->onAC.coveraction;
+ 			break;
+		case TAG_ONAC_KEYACTION:   /* sleepkey */
+			if (cfgure) {
+				if (taglist->data > ACTION_LAST)
+					tagerror(taglist, E_INVALID);
+				else
+					base->onAC.keyaction = taglist->data;
+			} else
+				taglist->data = base->onAC.keyaction;
+			break;	
+		case TAG_ONAC_TIMESUSPEND:
+		case TAG_ONACTIMESLEEP:   /* depreceated */
+			if (cfgure)	base->onAC.suspendtime = taglist->data;
+			else		taglist->data = base->onAC.suspendtime;
 			break;
-		case TAG_ONBATTERYCOVERSLEEP:
-			if (cfgure)	base->onBattery.flags.coversleep = taglist->data;
-			else		taglist->data = base->onBattery.flags.coversleep;
+		case TAG_ONAC_TIMEDIM:
+		case TAG_ONACTIMEDIM:     /* depreciated */
+			if (cfgure)	base->onAC.dimtime = taglist->data;
+			else		taglist->data = base->onAC.dimtime;
+ 			break;
+		case TAG_ONACSLEEP:       /* depreceated */
+			if (cfgure)	base->onAC.timeraction = taglist->data & 1 ? ACTION_TORAM : ACTION_BLANK;
+			else 		taglist->data = (base->onAC.timeraction == ACTION_TORAM || base->onAC.timeraction == ACTION_TODISK) ? 1 : 0;
 			break;
-		case TAG_ONBATTERYTIMESLEEP:
-			if (cfgure)	base->onBattery.Tsleep = taglist->data;
-			else		taglist->data = base->onBattery.Tsleep;
+		case TAG_ONACCOVERSLEEP:  /* depreceated */
+			if (cfgure)	base->onAC.coveraction = taglist->data & 1 ? ACTION_TORAM : ACTION_BLANK;
+			else 		taglist->data = (base->onAC.coveraction == ACTION_TORAM || base->onAC.coveraction == ACTION_TODISK) ? 1 : 0;
 			break;
-		case TAG_ONBATTERYTIMEDIM:
-			if (cfgure)	base->onBattery.Tdim = taglist->data;
-			else		taglist->data = base->onBattery.Tdim;
-			break;
-		case TAG_ONBATTERYPOLICY:
+	
+		case TAG_ONBATT_POLICY:	
+		case TAG_ONBATTERYPOLICY: /* depreciated */
 			if (cfgure) {
 				if (taglist->data > POLICY_LAST)
 					tagerror(taglist, E_INVALID);
@@ -338,6 +345,53 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 			} else	
 				taglist->data = base->onBattery.policy;
 			break;
+		case TAG_ONBATT_TIMERACTION:
+			if (cfgure) {
+				if (taglist->data > ACTION_LAST)
+					tagerror(taglist, E_INVALID);
+				else
+					base->onBattery.timeraction = taglist->data;
+			} else
+				taglist->data = base->onBattery.timeraction;
+			break;
+		case TAG_ONBATT_COVERACTION:
+			if (cfgure) {
+				if (taglist->data > ACTION_LAST)
+					tagerror(taglist, E_INVALID);
+				else
+					base->onBattery.coveraction = taglist->data;
+			} else
+				taglist->data = base->onBattery.coveraction;
+			break;
+		case TAG_ONBATT_KEYACTION:   /* sleepkey */
+			if (cfgure) {
+				if (taglist->data > ACTION_LAST)
+					tagerror(taglist, E_INVALID);
+				else
+					base->onBattery.keyaction = taglist->data;
+			} else
+				taglist->data = base->onBattery.keyaction;
+			break;
+		case TAG_ONBATT_TIMESUSPEND:
+		case TAG_ONBATTERYTIMESLEEP:  /* depreciated */
+			if (cfgure)	base->onBattery.suspendtime = taglist->data;
+			else		taglist->data = base->onBattery.suspendtime;
+			break;
+		case TAG_ONBATT_TIMEDIM:
+		case TAG_ONBATTERYTIMEDIM:    /* depreceated */
+			if (cfgure)	base->onBattery.dimtime = taglist->data;
+			else		taglist->data = base->onBattery.dimtime;
+			break;
+		case TAG_ONBATTERYSLEEP:      /* depreceated */
+			if (cfgure)	base->onBattery.timeraction = taglist->data & 1 ? ACTION_TORAM : ACTION_BLANK;
+			else 		taglist->data = (base->onBattery.timeraction == ACTION_TORAM || base->onBattery.timeraction == ACTION_TODISK) ? 1 : 0;
+			break;
+		case TAG_ONBATTERYCOVERSLEEP: /* depreceated */
+			if (cfgure)	base->onBattery.coveraction = taglist->data & 1 ? ACTION_TORAM : ACTION_BLANK;
+			else 		taglist->data = (base->onBattery.coveraction == ACTION_TORAM || base->onBattery.coveraction == ACTION_TODISK) ? 1 : 0;
+			break;
+		/* Tags for power profiles end */
+
 		case TAG_SLEEPKEY:
 			if (cfgure)	base->keysleep = taglist->data;
 			else		taglist->data = base->keysleep;
@@ -350,9 +404,12 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 			if (cfgure)	base->sleepkeydelaytime = taglist->data;
 			else		taglist->data = base->sleepkeydelaytime;
 			break;
-		case TAG_SLEEPKEYSLEEP:
-			if (cfgure)	base->flags.sleepkeysleep = taglist->data;
-			else		taglist->data = base->flags.sleepkeysleep;
+		case TAG_SLEEPKEYSLEEP:  /* depreceated */
+			if (cfgure) {
+				base->onAC.keyaction = taglist->data & 1 ? ACTION_TORAM : ACTION_BLANK;
+				base->onBattery.keyaction = taglist->data & 1 ? ACTION_TORAM : ACTION_BLANK;
+			} else 	
+				taglist->data = (base->onAC.keyaction == ACTION_TORAM || base->onAC.keyaction == ACTION_TODISK) ? 1 : 0;
 			break;
 		case TAG_BWLFIRST:
 			if (cfgure)	base->BWL1 = taglist->data;
@@ -441,7 +498,7 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 				if (base->mode == MODE_AWAKE)
 					base->sleeptime = 0;
 				else if (base->mode == MODE_DIM)
-					base->sleeptime = base->activeProfile->Tdim;
+					base->sleeptime = base->activeProfile->dimtime;
 				power_check_battery (base->timeremaining, base->powersource);
 			}
 			break;
@@ -452,7 +509,7 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 			}
 			break;
 		case TAG_PREPAREFORSLEEP:  /* private tag */
-			if (cfgure)	call_script (base->script_pmcs, "suspend", base->powersource ? "ac" : "battery");
+			/* PMCS-script will be called in power_suspend() so that it's nothing left to do here */
 			break;
 		case TAG_WAKEUPFROMSLEEP:  /* private tag */
 			if (cfgure) {
@@ -460,7 +517,7 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 				val = base->powersource;
 				power_sync ();  /* syncronise redundant data from module_pmac */
 				power_check_battery (-1, base->powersource);
-				call_script (base->script_pmcs, "resume", base->powersource ? "ac" : "battery");
+				call_script (base->script_pmcs, "resume", base->powersource ? "ac" : "battery", "ram");
 				/* if power source changed during sleep call PMCS script
 				 * again to set power policy */
 				if (val != base->powersource)
@@ -470,14 +527,22 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 		case TAG_COVERSTATUS:  /* private tag */
 			if (cfgure) {
 				base->flags.coveropen = taglist->data & 1;
-				if (base->flags.coveropen)
-					power_awake ();
-				else
-					power_trigger_sleep (base->activeProfile->flags.coversleep);
+				val = base->activeProfile->coveraction;
+				if (val == ACTION_BLANK || ((val == ACTION_TORAM) && !base->flags.sleep_supported))
+ 					call_script (base->script_pmcs, 
+ 						base->flags.coveropen ? "cover-open" : "cover-close",
+ 						base->powersource ? "ac" : "battery",
+						base->flags.coveropen ? "open" : "close");
+				if (base->flags.coveropen) 
+ 					power_awake ();
+				else {
+					power_suspend (ACTION_BLANK); /* always blank screen if cover has been closed */
+					power_suspend (val);          /* then follow user's will */
+				}
 			}
 			break;
 		case TAG_GOTOSLEEP:
-			if (cfgure)	power_trigger_sleep (base->flags.sleep_supported);
+			if (cfgure)	power_suspend (ACTION_TORAM);
 			else		tagerror (taglist, E_NOREAD);
 			break;
 		case TAG_POLICY:
@@ -508,14 +573,14 @@ power_timer (struct tagitem *taglist)
 	base->sleeptime++;
 	switch (base->mode) {
 	case MODE_AWAKE:
-		if ((base->activeProfile->Tdim) 
-		  && (base->sleeptime > base->activeProfile->Tdim)) { /* time to dim the display? */
+		if ((base->activeProfile->dimtime) 
+		  && (base->sleeptime > base->activeProfile->dimtime)) { /* time to dim the display? */
 			process_queue_single (CONFIGQUEUE, TAG_BRIGHTNESSOP, OP_DIM_LIGHT); /* then dim */
 			base->mode = MODE_DIM;
 		}
 	case MODE_DIM:
-		if ((base->activeProfile->Tsleep)
-		  && (base->sleeptime > base->activeProfile->Tsleep)
+		if ((base->activeProfile->suspendtime)
+		  && (base->sleeptime > base->activeProfile->suspendtime)
 		  && !base->flags.sleeplock_cpu
 		  && !base->flags.sleeplock_net) { /* time to sleep? */
 			base->mode = MODE_DOZY;
@@ -523,12 +588,14 @@ power_timer (struct tagitem *taglist)
 		}
 		break;
 	case MODE_DOZY:
-		n = --base->sleepcountdown / 10;       /* time left in seconds */
-		if (n == 0)
-			power_trigger_sleep (base->activeProfile->flags.sleep);
-		else
-			singletag_to_clients(WARNING, TAG_SLEEPINSECONDS, n);
-	case MODE_SLEEP:
+		if (base->activeProfile->timeraction != ACTION_NONE) {
+			n = --base->sleepcountdown / 10;       /* time left in seconds */
+			if (n == 0)
+				power_suspend (base->activeProfile->timeraction);
+			else
+				singletag_to_clients(WARNING, TAG_SLEEPINSECONDS, n);
+		}
+	case MODE_SUSPEND:
 		base->sleeptime--;
 		break;
 	}
@@ -655,7 +722,7 @@ power_check_battery (int time, int psource)
 								process_queue_single (CONFIGQUEUE, TAG_REQUESTSLEEP, 0);
 							} else {              /* otherwise use the last way out */
 								base->emergency = EMA_COMMAND;
-								call_script (base->script_pmcs, "emergency", base->powersource ? "ac" : "battery");
+								call_script (base->script_pmcs, "emergency", base->powersource ? "ac" : "battery", "");
 								sleep (10);
 							}
 						} else {
@@ -664,7 +731,7 @@ power_check_battery (int time, int psource)
 						}
 						break;
 					case EMA_COMMAND:
-						call_script (base->script_pmcs, "emergency", base->powersource ? "ac" : "battery");
+						call_script (base->script_pmcs, "emergency", base->powersource ? "ac" : "battery", "");
 						sleep(10);   /* give the script a chance to work */
 						break;
 					}
@@ -682,19 +749,35 @@ power_check_battery (int time, int psource)
     like enable_blank into account. */
 
 void
-power_trigger_sleep (int flag)
+power_suspend (int action)
 {
 	struct moddata_power *base = &modbase_power;
 
-	base->mode = MODE_SLEEP;
-	if (flag && base->flags.sleep_supported)
-		process_queue_single (CONFIGQUEUE, TAG_REQUESTSLEEP, 0);
-	else {
-		process_queue_single (CONFIGQUEUE, TAG_BRIGHTNESSOP, OP_DIM_OFF);
+	if (action == ACTION_NONE)  /* no action, mode will not change */
+		return;
+	
+	base->mode = MODE_SUSPEND;
+
+	switch (action) {
+	case ACTION_TODISK:
+		singletag_to_clients(WARNING, TAG_SLEEPINSECONDS, 0);
+		call_script (base->script_pmcs, "suspend", base->powersource ? "ac" : "battery", "disk");
+		break;
+	case ACTION_TORAM:
+		if (base->flags.sleep_supported) {
+			singletag_to_clients(WARNING, TAG_SLEEPINSECONDS, 0);
+/*			sleep(1);    give clients a chance to react */
+			call_script (base->script_pmcs, "suspend", base->powersource ? "ac" : "battery", "ram");
+			process_queue_single (CONFIGQUEUE, TAG_REQUESTSLEEP, 0);
+			break;
+		} /* else ACTION_BLANK */
+	case ACTION_BLANK:
+ 		process_queue_single (CONFIGQUEUE, TAG_BRIGHTNESSOP, OP_DIM_OFF);
 		if (base->flags.heartbeat_enable) {  /* user wants heartbeat beep ? */
 			base->flags.heartbeat = 1; /* enable heartbeat beep */
 			power_beep();
 		}
+		break;
 	}
 }
 
@@ -720,7 +803,7 @@ power_setpolicy (int policy, int powersource)
 		change = 1;
 	}
 	if (change)
-		call_script (base->script_pmcs, policies[base->policy], powersrc[base->powersource]);
+		call_script (base->script_pmcs, policies[base->policy], powersrc[base->powersource], "");
 }
 
 int
